SystemOrganization addCategory: #'SystemEditor-Editors'!
SystemOrganization addCategory: #'SystemEditor-Exceptions'!
SystemOrganization addCategory: #'SystemEditor-Info'!
SystemOrganization addCategory: #'SystemEditor-Tests'!

!ClassOrganizer methodsFor: '*SystemEditor-workaround' stamp: 'mtf 7/18/2008 16:22'!
edClassComment: aString stamp: aStamp
"Set the comment and stamp directly, with no checks.This works around a silly check in ClassOrganizer>>classComment:stamp: in Croquet. This check raises an error if the subject is either nil or a metaclass. The check does not exist in Squeak 3.8 and above"
	^ super classComment: aString stamp: aStamp! !

Object subclass: #AbstractEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!AbstractEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 6/27/2008 10:10'!
canEdit: anObject for: containerEditor
	^ false! !

!AbstractEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 6/27/2008 10:11'!
canEdit: anObject
	^ self canEdit: anObject for: nil! !

!AbstractEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 7/14/2008 11:16'!
named: aName for: aDictionaryEditor
	"Answers a new or existing instance with the given name, and installs it, if necessary"
	^ aDictionaryEditor at: aName ifAbsent: [
		(self on: nil for: aDictionaryEditor)
			setName: aName;
			edRegisterEditor;
			yourself]! !

!AbstractEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 9/25/2008 17:50'!
on: subject for: collectionEditor ifNotHandled: aBlock
	"Try to find the most specialized subclass willing to edit,  for extensibility"
	| editor |
	"nil subject is always permitted; it means new object"
	subject ifNil: [^ self new on: subject for: collectionEditor].
	self subclassesDo: [:class |
		editor := class on: subject for: collectionEditor ifNotHandled: [nil].
		editor ifNotNil: [^ editor]].
	(self canEdit: subject for: collectionEditor)
		ifTrue: [^ self new on: subject for: collectionEditor].
	^ aBlock value! !

!AbstractEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 7/11/2008 12:36'!
on: subject for: collectionEditor
	^ self on: subject for: collectionEditor ifNotHandled: [self error:
		self class name, ' does not know how to edit ', subject printString]! !

!AbstractEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 7/11/2008 12:36'!
on: subject ifNotHandled: aBlock
	^ self on: subject for: nil ifNotHandled: aBlock! !

!AbstractEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 6/27/2008 10:13'!
on: anObject
	^ self on: anObject for: nil! !

!AbstractEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:27'!
edIsEditor
	^ true! !

!AbstractEditor methodsFor: 'building' stamp: 'mtf 7/11/2008 12:47'!
edPrepareMigration: txn
	"Adds migrators to the given MigrationTransaction, as necessary to support converting my subject to my product"
	^ self! !

!AbstractEditor methodsFor: 'initialize-release' stamp: 'mtf 7/11/2008 13:00'!
edRegisterEditor
	"Register myself with my container, which is presumed to be a DictionaryEditor"
	self environment edRegisterEditor: self! !

!AbstractEditor methodsFor: 'accessing' stamp: 'mtf 10/6/2008 15:13'!
productClass
"Answers the class of the object that is the result of applying my edits to my subject"

	^ self subject class! !

!AbstractEditor methodsFor: 'accessing' stamp: 'mtf 10/6/2008 15:11'!
subject
"Answers the object I am editing"

	self subclassResponsibility! !

AbstractEditor subclass: #CategorizerEditor
	instanceVariableNames: 'subject renamedCategories categories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!CategorizerEditor commentStamp: '<historical>' prior: 0!
An abstract superclass for editors of Categorizers. Implements the public interface of a Categorizer. I maintain the list of added, removed, or reordered categories, but it is up to subclasses to remember which elements were added, removed, or recategorized.

subject (Categorizer) - The Categorizer I am editing
categories (OrderedCollection) - The ordered list of categories
renamedCategories (Dictionary) - A dictionary mapping old category names to new names!

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/6/2008 19:13'!
addCategory: catString before: nextCategory
	"I don't support ordering"
	| newCategory |
	newCategory := catString asSymbol.
	(self categories includes: newCategory) ifTrue: [^ self].
	categories add: newCategory before: nextCategory! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/11/2007 19:10'!
addCategory: newCategory
	^ self addCategory: newCategory before: nil! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/12/2007 10:56'!
allMethodSelectors
	"give a list of all method selectors."

	^ self elementArray copy sort! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/6/2008 21:15'!
categories

	^ categories! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/6/2008 21:16'!
categories: anArray

	categories := anArray! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:38'!
categoryOfChange: element
"Answers the category of a new or recategorized element"

	self changedElementsAndCategoriesDo: [:anElement :aCategory |
		element = anElement ifTrue: [^ aCategory]].
	^ nil! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 1/23/2008 16:55'!
categoryOfElement: element
	(self removedElements includes: element) ifTrue: [^ nil].
	(self categoryOfChange: element) ifNotNilDo: [:cat | ^ cat].
	self subject ifNil: [^ nil].
	^ self newCategoryFor: (self subject categoryOfElement: element)
! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:40'!
changedCategories
"Answer the set of categories whose elements have changed"

	^ (self changedElementsAndCategoriesCollect:
		[:anElement :aCategory | aCategory]) asSet
! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:45'!
changedElementsAndCategoriesCollect: aBlock
"Evaluates aBlock for each added or recategorized elements, and its category. Answers an OrderedCollection of the results"

	| newCollection |
	newCollection := OrderedCollection new.
	self changedElementsAndCategoriesDo: [:anElement :aCategory |
		newCollection add: (aBlock value: anElement value: aCategory)].
	^ newCollection! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:45'!
changedElementsAndCategoriesDo: aBlock
"Evaluates aBlock for each added or recategorized elements, and its category"

	self subclassResponsibility! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:52'!
changedElementsAndCategoriesSelect: selectBlock thenCollect: collectBlock
"Evaluates collectBlock for each added or recategorized element and its category for which selectBlock answers true. Answers an OrderedCollection of the results"

	| newCollection |
	newCollection := OrderedCollection new.
	self changedElementsAndCategoriesDo: [:anElement :aCategory |
		(selectBlock value: anElement value: aCategory) ifTrue: [newCollection
			add: (collectBlock value: anElement value: aCategory)]].
	^ newCollection! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:41'!
changedElements
"Answers the list of elements that have been added or recategorized"

	^ self changedElementsAndCategoriesCollect: [:anElement :aCategory | anElement]! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:46'!
changesInCategory: category
"Answers a list of new or recategorized elements in category"

	^ self changedElementsAndCategoriesSelect:
			[:anElement :aCategory | category = aCategory]
		thenCollect: [:anElement :aCategory | anElement]! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/11/2007 19:45'!
classifyAll: aCollection under: heading

	aCollection do:
		[:element | self classify: element under: heading]! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:18'!
classify: element under: heading suppressIfDefault: aBoolean
	| category |
	((heading = Categorizer nullCategory) or: [heading == nil])
		ifTrue: [category := Categorizer default]
		ifFalse: [category := heading asSymbol].
	self addCategory: category.
	((self categories includes: Categorizer default)
		and: [(self listAtCategoryNamed: Categorizer default) size = 0])
			ifTrue: [self removeCategory: Categorizer default].
! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/11/2007 19:44'!
classify: element under: heading 
	self classify: element under: heading suppressIfDefault: true! !

!CategorizerEditor methodsFor: 'building' stamp: 'mtf 10/7/2008 10:53'!
edBuild
	| result processed newCat |
	result := self productClass defaultList: Array new.
	result categories: categories.
	self changedElementsAndCategoriesDo: [:ea :cat |
		result classify: ea under: cat].
	processed := IdentitySet new 
		addAll: result elementArray;
		addAll: self removedElements;
		yourself.
	self subject categories do: [:oldCat |
		newCat := self newCategoryFor: oldCat.
		(subject listAtCategoryNamed: oldCat) do: [:ea |
			(processed includes: ea) ifFalse: [
				result classify: ea under: newCat]]].
	^ result! !

!CategorizerEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 22:03'!
edPrepareMigration: txn
	self edRequiresBuild
		ifTrue: [txn addMigrator: (ObjectMigrator 
									origin: self subject 
									destination: self edBuild)]! !

!CategorizerEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 14:58'!
edRequiresBuild
	^ self hasChangedElements or: [self removedElements notEmpty]! !

!CategorizerEditor methodsFor: 'reflecting - private' stamp: 'mtf 9/12/2007 10:55'!
elementArray
	^ self categories gather: [:category | self listAtCategoryNamed: category]! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 14:56'!
hasChangedElements
	self changedElementsAndCategoriesDo: [:element :category | ^ true].
	^ false! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:08'!
listAtCategoryNamed: catName

	| category list |
	category := catName asSymbol.
	(self categories includes: category) ifFalse: [^ nil].
	list := self subject ifNotNil: [self subject listAtCategoryNamed: category].
	list ifNil: [list := OrderedCollection new].
	list := list asOrderedCollection.
	self removedElements do: [:ea | list removeAll: ea].
	self changedElementsAndCategoriesDo: [:ea :cat |
		list removeAll: ea. cat = category ifTrue: [list add: ea]].
	^ list
! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/11/2007 19:59'!
listAtCategoryNumber: anInteger
	^ self listAtCategoryNamed: (self categories at: anInteger)! !

!CategorizerEditor methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 19:14'!
newCategoryFor: oldCategory
	^ renamedCategories at: oldCategory ifAbsent: [oldCategory]! !

!CategorizerEditor methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 19:14'!
oldCategoryFor: newCategory
	^ renamedCategories keyAtValue: newCategory ifAbsent: [newCategory]! !

!CategorizerEditor methodsFor: 'reflecting - printing' stamp: 'mtf 9/11/2007 19:45'!
printOn: aStream
	self categories do: [:category |
		aStream nextPutAll: '('''; nextPutAll: category; nextPut: $'.
		(self listAtCategoryNamed: category) do: [:element |
			aStream space; nextPutAll: element].
		aStream nextPut: $); cr]! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:19'!
removeCategory: catString
	| category |
	category := catString asSymbol.
	(self categories includes: category) ifFalse: [^ self].
	(self listAtCategoryNamed: category)
		ifNotEmpty: [^self error: 'cannot remove non-empty category'].
	categories remove: category ifAbsent: [].
	renamedCategories keysAndValuesRemove: [:key :value | value == category].
! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:47'!
removedElements
"Answers a list of elements that have been removed"

	^ Array new! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:24'!
removeElement: element
	self subclassResponsibility! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:24'!
renameCategory: oldCatString toBe: newCatString
	| oldCategory newCategory originalCategory |
	newCategory := newCatString asSymbol.
	oldCategory := oldCatString asSymbol.
	originalCategory := self oldCategoryFor: oldCategory.
	(self categories includes: newCategory) ifTrue: [^ self].
	(self categories includes: oldCategory) ifFalse: [^ self].
	categories replaceAll: oldCategory with: newCategory.
	renamedCategories at: originalCategory put: newCategory.! !

!CategorizerEditor methodsFor: 'initialize-release' stamp: 'mtf 10/7/2008 10:11'!
setSubject: aCategorizer
	subject := aCategorizer.
	renamedCategories := Dictionary new.
	categories := OrderedCollection new.! !

!CategorizerEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 19:07'!
subject
	^ subject! !

CategorizerEditor subclass: #OrganizationEditor
	instanceVariableNames: 'classEditor isDirty comment stamp remote repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!OrganizationEditor commentStamp: 'mtf 7/14/2008 11:28' prior: 0!
My instances are editors for ClassOrganizer instances!

!OrganizationEditor class methodsFor: 'instance creation' stamp: 'cwp 12/18/2005 14:47'!
for: aClassEditor repository: aSourceRepository
	^ self new setClassEditor: aClassEditor repository: aSourceRepository! !

!OrganizationEditor class methodsFor: 'instance creation' stamp: 'cwp 12/18/2005 14:47'!
for: aClassEditor
	^ self for: aClassEditor repository: SourceRepository default! !

!OrganizationEditor class methodsFor: 'instance creation' stamp: 'cwp 12/15/2005 22:20'!
on: anOrganizer
	^ self for: anOrganizer subject! !

!OrganizationEditor methodsFor: 'accessing' stamp: 'mtf 1/24/2008 20:57'!
changedElementsAndCategoriesDo: aBlock
	classEditor methods additions keysAndValuesDo: [:key :editor |
		aBlock value: key value: editor category]! !

!OrganizationEditor methodsFor: 'SystemEditor-Editors' stamp: 'mtf 9/11/2007 17:06'!
classComment
	^comment ifNil: [self subject ifNotNil: [self subject classComment]]! !

!OrganizationEditor methodsFor: 'reflecting' stamp: 'cwp 2/1/2006 22:49'!
classComment: aStringOrText stamp: aString
	comment := aStringOrText.
	stamp := aString.
	isDirty := true
	! !

!OrganizationEditor methodsFor: 'SystemEditor-Editors' stamp: 'mtf 9/11/2007 17:06'!
commentRemoteStr
	"If we are dirty the comment has been explicitly set and if
	it was set to something else than nil we lazily create the remote String."

	(isDirty and: [comment notNil] and: [remote isNil]) ifTrue: [self writeComment].
	^ remote ifNil: [self subject ifNotNil: [self subject commentRemoteStr]]! !

!OrganizationEditor methodsFor: 'reflecting' stamp: 'mtf 9/11/2007 17:07'!
commentStamp
	^stamp ifNil: [self subject ifNotNil: [self subject commentStamp]]! !

!OrganizationEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 22:05'!
edBuild
	| result |
	result := super edBuild.
	result
		edClassComment: self commentRemoteStr
		stamp: self commentStamp.
	^ result! !

!OrganizationEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 22:08'!
edRequiresBuild
	^ isDirty or: [super edRequiresBuild]! !

!OrganizationEditor methodsFor: 'accessing' stamp: 'mtf 1/25/2008 19:45'!
removedElements
	^ classEditor methods removals! !

!OrganizationEditor methodsFor: 'initialize-release' stamp: 'mtf 9/11/2007 17:13'!
setClassEditor: aClassEditor repository: aSourceRepository
	isDirty := false.
	classEditor := aClassEditor.
	repository := aSourceRepository.
	self setSubject: nil.! !

!OrganizationEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 17:17'!
subject
	^ subject ifNil: [classEditor subject ifNotNilDo: [:aClass | subject := aClass organization]]! !

!OrganizationEditor methodsFor: 'building' stamp: 'mtf 8/29/2007 14:59'!
writeComment
	| previous subject |
	subject := self subject.
	previous := subject ifNotNil: [subject commentRemoteStr].
	remote := previous 
				ifNil: [repository 
						storeComment: comment
						forClass: classEditor
						stamped: stamp]
				ifNotNil: [repository 
							storeComment: comment
							forClass: classEditor
							stamped: stamp
							replacing: previous].! !

CategorizerEditor subclass: #SystemOrganizationEditor
	instanceVariableNames: 'system'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!SystemOrganizationEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 22:08'!
for: anEnvironment
	^ self new setSystem: anEnvironment! !

!SystemOrganizationEditor methodsFor: 'accessing' stamp: 'mtf 1/24/2008 19:04'!
changedElementsAndCategoriesDo: aBlock
	system editors keysAndValuesDo: [:key :editor |
		aBlock value: key value: editor category]! !

!SystemOrganizationEditor methodsFor: 'accessing' stamp: 'mtf 1/24/2008 20:54'!
removedElements
	^ system removals! !

!SystemOrganizationEditor methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 22:09'!
setSystem: anEnvironment
	system := anEnvironment! !

!SystemOrganizationEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 22:13'!
subject
	^ subject ifNil: [self setSubject: system subject organization. subject]! !

CategorizerEditor subclass: #TestCategoryEditor
	instanceVariableNames: 'changes removedElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!TestCategoryEditor commentStamp: 'mtf 9/11/2007 12:54' prior: 0!
A editor for Categorizers. I don't preserve order of changed categories. changes is a categorizer that handles all changes!

!TestCategoryEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 15:21'!
on: aCategorizer
	^ self new setSubject: aCategorizer! !

!TestCategoryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 20:03'!
categoryOfChange: element
	^ changes categoryOfElement: element! !

!TestCategoryEditor methodsFor: 'private' stamp: 'mtf 9/11/2007 19:19'!
changedElements
	^ changes elementArray! !

!TestCategoryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 20:03'!
changesAddCategory: newCategory before: nextCategory
	"I don't support ordering"
	changes addCategory: newCategory! !

!TestCategoryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 19:58'!
changesInCategory: category
	^ changes listAtCategoryNamed: category! !

!TestCategoryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 20:01'!
changesRemoveCategory: category
	changes removeCategory: category! !

!TestCategoryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 19:49'!
changesRemoveElement: element
	changes removeElement: element! !

!TestCategoryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 19:47'!
changesRenameCategory: oldCategory toBe: newCategory
	changes renameCategory: oldCategory toBe: newCategory! !

!TestCategoryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 20:02'!
classifyChange: element under: category suppressIfDefault: aBoolean
	changes classify: element under: category suppressIfDefault: aBoolean! !

!TestCategoryEditor methodsFor: 'private' stamp: 'mtf 9/11/2007 19:18'!
removedElements
	^ removedElements ! !

!TestCategoryEditor methodsFor: 'initialize-release' stamp: 'mtf 9/11/2007 20:14'!
setSubject: aCategorizer
	super setSubject: aCategorizer.
	changes := Categorizer defaultList: Array new.
	removedElements := Set new! !

AbstractEditor subclass: #ClassDescriptionEditor
	instanceVariableNames: 'superclass methodDict format superEditor methods type instVarNames organization properties decorators'
	classVariableNames: 'ReservedNames'
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ClassDescriptionEditor commentStamp: 'mtf 7/29/2008 21:55' prior: 0!
I am an abstract superclass for both ClassEditor and MetaclassEditor.

I implement  the Behavior protocol: my instances can create instances. This means that my first three variables must remain 'superclass methodDict format', as the VM accesses them positionally, and my superclasses must not define any instance vairables. Only MetaclassEditor makes use of these ivars, but they are defined here to avoid duplicating a lot of ivars and accessing methods between ClassEditor and MetaclassEditor!

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:38'!
acceptsLoggingOfCompilation

	self subject ifNil: [ ^ true ].
	^ self subject acceptsLoggingOfCompilation! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 22:59'!
addInstVarName: aString 
	instVarNames ifNil: [instVarNames := self subject instVarNames].
	(instVarNames includes: aString)
		ifFalse: [instVarNames := instVarNames copyWith: aString]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:53'!
allInstVarNames
	^ self superclassOrEditor allInstVarNames , self instVarNames! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:01'!
allSuperclasses
"Answer an OrderedCollection of the receiver's and the receiver's ancestor's superclasses. The first element is the receiver's immediate superclass, followed by its superclass; the last element is Object."

	^ self superclass withAllSuperclasses! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:01'!
browserDefinition: style
	^ self definition! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:04'!
classSide
	^self theMetaClass! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:13'!
compiledMethodAt: aSelector ifAbsent: aBlock
	^ self edMethodAt: aSelector ifAbsent: aBlock! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:12'!
compiledMethodAt: aSelector

	^ self compiledMethodAt: aSelector ifAbsent: [ self error: 'key not found' ]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:14'!
compilerClass
	^ self subject 
		ifNil: [Compiler]
		ifNotNil: [self subject compilerClass]! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:01'!
compile: source classified: cat notifying: anObject
	^ self 
		compile: source
		classified: cat
		withStamp: Utilities changeStamp
		notifying: anObject
		logSource: self acceptsLoggingOfCompilation ! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:09'!
compile: source classified: cat withStamp: aString notifying: anObject logSource: aBoolean
	| cm |
	cm := MethodEditor 
			source: source
			classified: cat
			stamp: aString
			notifying: anObject
			logging: aBoolean.
	self methods add: cm.
	^ cm selector! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:06'!
compile: source classified: cat withStamp: aString notifying: anObject
	^ self 
		compile: source
		classified: cat
		withStamp: aString
		notifying: anObject
		logSource: self acceptsLoggingOfCompilation ! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:00'!
compile: source classified: cat
	^ self 
		compile: source
		classified: cat
		withStamp: Utilities changeStamp
		notifying: nil
		logSource: self acceptsLoggingOfCompilation ! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 7/29/2008 22:40'!
decoratorsDo: aBlock
	decorators ifNotNil: [decorators do: aBlock]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:15'!
definitionST80: isST80
	^ isST80 ifTrue: [self definitionST80] ifFalse: [self definition]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:15'!
definition
	^ self definitionST80! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 7/29/2008 08:59'!
edClassFormat
	^ ClassFormat
		size: self instSize
		type: self typeOfClass
		index: self indexIfCompact! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:11'!
edMethodAt: aSelector ifAbsent: aBlock
	^ self methods at: aSelector ifAbsent: aBlock! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:10'!
edMethodAt: aSelector
	^ self methods at: aSelector! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 7/29/2008 22:40'!
edMethodsDo: aBlock
	methods ifNotNil: [methods do: aBlock]! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 7/29/2008 08:57'!
edOldFormat
	^ self subject ifNotNil: [ClassFormat fromBits: self subject format]! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 9/26/2008 11:23'!
edSuperclass
"Answer my product's superclass, building it if necessary"

	| classOrEditor |
	classOrEditor :=  self superclassOrEditor.
	^ classOrEditor edIsEditor
		ifTrue: [classOrEditor product]
		ifFalse: [classOrEditor]! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:11'!
ensureInstVarName: aString 
	(self allInstVarNames includes: aString)
		ifFalse: [ self addInstVarName: aString].
		 ! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:17'!
format
	^ self edClassFormat bits! !

!ClassDescriptionEditor methodsFor: 'accessing properties' stamp: 'mtf 7/29/2008 20:28'!
hasProperty: aSymbol
	self propertyAt: aSymbol ifAbsent: [^ false].
	^ true! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:18'!
includesSelector: aSelector
	^ (methods ifNil: [self subject] ifNotNil: [methods])
		includesSelector: aSelector! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:19'!
indexIfCompact
	self subject ifNil: [^ 0].
	^ self subject indexIfCompact! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:23'!
instanceSide
	^ self theNonMetaClass! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:24'!
instanceVariableNames: instString
	instVarNames := Scanner new scanFieldNames: instString! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:25'!
instanceVariablesString
	^ String streamContents: 
		[:stream | 
		self allInstVarNames
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream nextPut: $ ]]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:20'!
instSize
	^ self instVarNames size + self superclassOrEditor instSize! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 9/19/2008 12:08'!
instVarNames
	^ instVarNames ifNil: [self subject instVarNames]! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:33'!
isBits
	^ #(bytes words) includes: self typeOfClass! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:33'!
isBytes
	^ self typeOfClass = #bytes! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 10/6/2008 13:47'!
isMethodsChanged
	methods ifNil: [^ false].
	^ methods isChanged! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:34'!
isPointers
	^ #(normal weak variable) includes: self typeOfClass! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:34'!
isVariable
	^ (self typeOfClass = #normal) not! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:34'!
isWeak
	^ self typeOfClass = #weak! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:34'!
isWords
	^ #(words weak variable) includes: self typeOfClass ! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:36'!
kindOfSubclass
	^ self typeOfClass caseOf: {
		[#normal] -> [' subclass: '].
		[#words] -> [' variableWordSubclass: '].
		[#variable] -> [' variableSubclass: '].
		[#bytes] -> [' variableByteSubclass: '].
		[#weak] -> [' weakSubclass: ' ].
		[#compiledMethod] -> [' variableByteSubclass: ' ]
	} otherwise: [self error: 'Unrecognized class type']! !

!ClassDescriptionEditor methodsFor: 'compiling' stamp: 'mtf 7/29/2008 21:36'!
literalScannedAs: literal notifying: anEncoder 
	literal isVariableBinding ifTrue: [self halt].
	^ literal! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 7/29/2008 22:39'!
methods
	^ methods ifNil: [methods := MethodDictionaryEditor for: self]! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 8/6/2008 00:37'!
organization
	^ organization ifNil: [organization := OrganizationEditor for: self]! !

!ClassDescriptionEditor methodsFor: 'accessing properties' stamp: 'mtf 7/29/2008 20:27'!
propertyAt: key ifAbsentPut: aBlock

	^ self propertyAt: key ifAbsent: [
		self propertyAt: key put: aBlock value]! !

!ClassDescriptionEditor methodsFor: 'accessing properties' stamp: 'mtf 7/29/2008 22:33'!
propertyAt: key ifAbsent: aBlock

	properties ifNil: [^ aBlock value].
	^ properties at: key ifAbsent: aBlock! !

!ClassDescriptionEditor methodsFor: 'accessing properties' stamp: 'mtf 7/29/2008 22:34'!
propertyAt: aSymbol put: value

	properties ifNil: [properties := Dictionary new].
	^ properties at: aSymbol put: value! !

!ClassDescriptionEditor methodsFor: 'accessing properties' stamp: 'mtf 7/29/2008 20:24'!
propertyAt: key

	^self propertyAt: key ifAbsent: [nil]! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:16'!
rearrangeInstVarNames: associations
	"The associations are name -> pos and cover the subset
	of vars that should be moved. The old just fill in the holes."

	instVarNames := ClassEditor rearrangeVarNames: associations oldNames: instVarNames! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:22'!
removeInstVarName: aString 
	instVarNames ifNil: [instVarNames := self subject instVarNames].
	instVarNames := instVarNames copyWithout: aString! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:25'!
removeSelector: aSymbol 
	self methods remove: aSymbol.! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:44'!
selectors
	^ self organization allMethodSelectors! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:45'!
sourceCodeAt: aSelector ifAbsent: aBlock
	^ (self edMethodAt: aSelector ifAbsent: aBlock) source! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:44'!
sourceCodeAt: aSelector
	^ self sourceCodeAt: aSelector ifAbsent: [nil]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:46'!
sourceCodeTemplate
	^ self class sourceCodeTemplate! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 8/5/2008 22:57'!
superclassOrEditor
"Answer my superclass as either an editor or class, whichever is easier. Does not add anything to my SystemEditor"

	^ superEditor
		ifNil: [ self subject superclass
			ifNil: [RootClassEditor on: nil for: self environment]]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:50'!
superclass
"Answers the editor on my new superclass, creating it if necessary"

	^ self environment edEditorFor: self superclassOrEditor! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:52'!
superclass: aClassOrEditor 
	superEditor := self environment edEditorFor: aClassOrEditor! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 8/6/2008 01:01'!
typeOfClass
	type ifNotNil: [^ type].
	self subject ifNotNil: [^ self subject typeOfClass].
	^ self superclassOrEditor typeOfClass.! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 8/5/2008 23:52'!
typeOfClass: aSymbol
	type := aSymbol! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:57'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
"Return the source code at selector, deferring to superclass if necessary"

	^ self sourceCodeAt: selector ifAbsent:
		[self superclassOrEditor ultimateSourceCodeAt: selector ifAbsent: aBlock]! !

!ClassDescriptionEditor methodsFor: 'validating' stamp: 'mtf 7/25/2008 22:56'!
validateFormat
	self edClassFormat
		validate;
		validateAgainstSuper: self edSuperFormat;
		validateAgainstOld: self edOldFormat! !

!ClassDescriptionEditor methodsFor: 'validating' stamp: 'mtf 9/19/2008 11:45'!
validateInstVarNames
	self instVarNames do: 
		[ :ea | 
		(ReservedNames includes: ea) ifTrue: [ IllegalVariableName signal ].
		(self superclassOrEditor allInstVarNames includes: ea) ifTrue: [ IllegalVariableName signal ].
		self subclassesOrEditors do: [ :class | (class instVarNames includes: ea) ifTrue: [ IllegalVariableName signal ] ] ]! !

!ClassDescriptionEditor methodsFor: 'validating' stamp: 'mtf 7/25/2008 22:56'!
validate
	self
		validateFormat;
		validateInstVarNames! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:58'!
whichCategoryIncludesSelector: aSymbol 
	^ self organization categoryOfElement: aSymbol! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:57'!
withAllSuperclasses
	"Answer an OrderedCollection of the receiver and the receiver's 
	superclasses. The first element is the receiver, 
	followed by its superclass; the last element is Object."

	| temp |
	temp := self allSuperclasses.
	temp addFirst: self.
	^ temp! !

ClassDescriptionEditor subclass: #ClassEditor
	instanceVariableNames: 'subject product system name classVarNames sharedPools category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ClassEditor class methodsFor: 'instance creation' stamp: 'mtf 7/11/2008 00:07'!
canEdit: anObject for: anEditor
	^ anObject isBehavior! !

!ClassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:19'!
classMetaclassEditor
"Answer the class of editors for my instances' cooresponding metaclass editor"

	^ OldMetaclassEditor! !

!ClassEditor class methodsFor: '*SystemEditor-Tests' stamp: 'cwp 11/6/2005 23:17'!
forNewClassNamed: aSymbol
	^ self named: aSymbol for: SystemEditor new! !

!ClassEditor class methodsFor: 'instance creation' stamp: 'mtf 7/29/2008 00:21'!
new
	^ self classMetaclassEditor new new! !

!ClassEditor class methodsFor: 'util' stamp: 'gk 9/3/2007 23:41'!
rearrangeVarNames: associations oldNames: oldVars
	"Rearrange vars. The associations are name -> pos and cover the subset
	of vars that should be moved. The old ones just fill in the holes."

	| newVars |
	newVars := Array new: oldVars size.
	associations do: [:assoc |
		newVars at: assoc value put: assoc key].
	oldVars do: [:oldVar |
		(newVars includes: oldVar) ifFalse: [
			newVars at: (newVars indexOf: nil) put: oldVar]].
	^newVars! !

!ClassEditor class methodsFor: 'accessing' stamp: 'cwp 11/20/2005 18:34'!
reservedNames
	^ ReservedNames! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/24/2005 08:27'!
addClassVarName: aSymbol
	classVarNames ifNil: [classVarNames := subject classVarNames].
	classVarNames := classVarNames copyWith: aSymbol asSymbol! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 16:03'!
addSharedPool: aSharedPool 
	sharedPools ifNil: [sharedPools := subject sharedPools].
	sharedPools := sharedPools copyWith: aSharedPool! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 6/26/2008 11:25'!
binding
	^ system bindingOf: self name! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 2/1/2008 21:12'!
category
	"If this isn't cached, SystemEditorBrowser is really slow. And it doesn't affect the correctness of the editor, so do it by default"
	^ category ifNil: [category := subject category]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/3/2005 09:41'!
category: aSymbol
	category := aSymbol! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 12/11/2005 22:36'!
classComment: aTextOrString stamp: aString
	self organization classComment: aTextOrString stamp: aString! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 11/4/2005 08:23'!
classPool
	| pool |
	pool := subject ifNil: [Dictionary new] ifNotNil: [subject classPool copy].
	classVarNames ifNil: [^ pool].
	pool keys do: [:ea | (classVarNames includes: ea) ifFalse: [pool removeKey: ea]].
	classVarNames do: [:ea | (pool includesKey: ea) ifFalse: [pool at: ea put: nil]].
	^ pool! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/2/2005 22:58'!
classVariablesString
	^ String streamContents: 
		[:stream | 
		self classVarNames asSortedCollection
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream nextPut: $ ]]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/23/2005 15:41'!
classVarNames
	^ classVarNames ifNil: [subject classVarNames]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/11/2005 22:09'!
comment
	| comment |
	comment := self organization classComment.
	^ comment isEmpty
		ifFalse: [comment]
		ifTrue: [self edCommentTemplate]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 2/1/2008 22:44'!
definitionST80
	"Answer a String that defines the receiver."

	| aStream |
	aStream := WriteStream on: (String new: 300).
	self superclass == nil
		ifTrue: [aStream nextPutAll: 'ProtoObject']
		ifFalse: [aStream nextPutAll: self  superclass name].
	aStream nextPutAll: self kindOfSubclass;
			store: self name.
	aStream cr; tab; nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString.
	aStream cr; tab; nextPutAll: 'classVariableNames: ';
			store: self classVariablesString.
	aStream cr; tab; nextPutAll: 'poolDictionaries: ';
			store: self sharedPoolsString.
	aStream cr; tab; nextPutAll: 'category: ';
			store: (SystemOrganization categoryOfElement: self name) asString.

	self superclass ifNil: [ 
		aStream nextPutAll: '.'; cr.
		aStream nextPutAll: self name.
		aStream space; nextPutAll: 'superclass: nil'. ].

	^ aStream contents! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 12:03'!
edBuild
	| meta |
	meta := self class edBuild.
	product := subject ifNil: [meta new]
				ifNotNil: [meta adoptInstance: subject from: subject class].
	product
		superclass: self edSuperclass;
		setFormat: self format;
		setName: self name;
		setInstVarNames: self instVarNames;
		classPoolFrom: self;
		instVarNamed: #sharedPools put: self sharedPools;
		organization: self organization build.
	"Create a temporary MethodDictionary to catch code written by SyntaxError dialogs. MethodDictionaryEditor will overwrite this. See MethodEditor>>compileFor:"
	product methodDictionary: MethodDictionary new.
	self decoratorsDo: [:ea | ea buildFor: self].

	"Class methods should be compiled before instance methods,
	since #compilerClass may be among the class methods"
	self class methods buildFor: self class.
	self methods buildFor: self.
	^product! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 12/3/2005 18:01'!
edCategory
	^ category! !

!ClassEditor methodsFor: 'accessing' stamp: 'cwp 12/11/2005 22:07'!
edCommentTemplate
	^ 'Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.'! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 9/20/2008 06:52'!
edExpand
	^ self edRequiresSubclassRebuild 
		ifTrue: [self subclasses]
		ifFalse: [#()]! !

!ClassEditor methodsFor: '*SystemEditor-Tests' stamp: 'cwp 11/20/2005 18:09'!
edIsValid
	[self validate]
		on: InvalidSystemChange
		do: [:ex | ^ false].
	^ true! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 9/4/2007 22:21'!
edNameHasChanged
	name ifNil: [^ false].
	^ name ~~ self subject name! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 7/22/2008 12:09'!
edName: aSymbol superclassEditor: anEditor type: typeOfClass instVarString: instString classVarString: classString poolImports: poolString category: catString 
	| scanner |
	scanner := Scanner new.
	name := aSymbol.
	self superclass: anEditor.
	type := typeOfClass.
	category := catString.
	instVarNames := scanner scanFieldNames: instString.
	classVarNames := scanner scanFieldNames: classString.
	sharedPools := (scanner scanFieldNames: poolString) 
				collect: [:ea | system edResolve: ea asSymbol]! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 3/21/2006 15:36'!
edPrepareExport: exporter 
	^exporter addClass: self product! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 4/15/2006 20:19'!
edPrepareInstanceMigration: txn
	txn addMigrator: (InstanceMigrator from: self subject to: self product)! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 6/20/2008 09:24'!
edPrepareMigration: txn
	self subject ifNil: [^ self].
	self edRequiresBuild 
		ifTrue: [self edPrepareInstanceMigration: txn]
		ifFalse: [methods edPrepareMigration: txn for: self.
				organization edPrepareMigration: txn.
				self decoratorsDo: [:ea | ea edPrepareMigration: txn]]! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 2/1/2006 21:40'!
edRequiresBuildFor: anObject
	^ anObject
		ifNil: [false]
		ifNotNil: [anObject edRequiresBuild]! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 6/20/2008 22:32'!
edRequiresBuild
	self edRequiresRecompile ifTrue: [^ true].
	self edSuperclassHasChanged ifTrue: [^ true].
	self edNameHasChanged ifTrue: [^ true].
	(self edRequiresBuildFor: self class) ifTrue: [^ true].
	self decoratorsDo: [:ea | ea edRequiresBuild ifTrue: [^ true]].
	^ false! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 11/7/2005 19:54'!
edRequiresMigration
	^ subject
		ifNotNil: [self format ~= subject format]
		ifNil: [false]! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 6/20/2008 22:09'!
edRequiresRecompile
	instVarNames ifNotNil: [^ true].
	classVarNames ifNotNil: [^ true].
	sharedPools ifNotNil: [^ true].
	self decoratorsDo: [:ea | ea edRequiresRecompile ifTrue: [^ true]].
	^ false! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 11/7/2005 20:00'!
edRequiresSubclassRebuild
	^ self class edRequiresSubclassRebuild 
		or: [self edRequiresMigration]
		or: [self edRequiresRecompile]! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 2/1/2006 21:50'!
edSuperclassHasChanged
	^ superEditor ~~ nil! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 7/28/2008 13:46'!
edSuperFormat
	^ self superclassOrEditor 
		ifNotNilDo: [ :ed | ClassFormat fromBits: ed format ]
		ifNil: [ ClassFormat named ]! !

!ClassEditor methodsFor: 'editing' stamp: 'kph 5/30/2007 16:47'!
ensureClassVarName: aString 
	(self allClassVarNames includes: aString)
		ifFalse: [ self addClassVarName: aString].
		 ! !

!ClassEditor methodsFor: 'editing' stamp: 'kph 5/30/2007 16:48'!
ensureSharedPoolName: aString 
	(self allSharedPools includes: aString)
		ifFalse: [ self addSharedPoolName: aString].
		 ! !

!ClassEditor methodsFor: 'accessing' stamp: 'mtf 11/10/2007 00:45'!
environment
	^ self system! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/11/2005 13:32'!
hasComment
	| comment |
	comment := self organization classComment.
	^ comment notNil and: [comment isEmpty not]! !

!ClassEditor methodsFor: 'debugging' stamp: 'mtf 9/4/2007 20:01'!
instVarAt: index put: anObject
	"Override the implemenation in Object. That will set the inst vars of 
	ClassEditor, but we want the inst vars of the subject."

	self class isDebuggingAsEditor ifTrue: [^ super instVarAt: index put: anObject].
	
	"If the insance variable has not yet been installed, return nil"
	self subject ifNil: [^ self].
	index > self subject class instSize ifTrue: [^ self].
	
	^ self subject instVarAt: index put: anObject! !

!ClassEditor methodsFor: 'debugging' stamp: 'mtf 9/4/2007 20:01'!
instVarAt: index
	"Override the implemenation in Object. That will return the inst vars of 
	ClassEditor, but we want the inst vars of the subject."

	self class isDebuggingAsEditor ifTrue: [^ super instVarAt: index].
	
	"If the insance variable has not yet been installed, return nil"
	self subject ifNil: [^ nil].
	index > self subject class instSize ifTrue: [^ nil].
	
	^ self subject instVarAt: index! !

!ClassEditor methodsFor: 'testing' stamp: 'mtf 9/13/2007 19:54'!
isMeta
	^ false! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 9/11/2007 23:41'!
methodDictionary
	| methodEditor |
	(methods isNil and: [self edRequiresRecompile not]) ifTrue: [^ subject methodDictionary].
	methodEditor := methods ifNil: [MethodDictionaryEditor for: self].
	^ methodEditor buildFor: self! !

!ClassEditor methodsFor: 'accessing' stamp: 'gk 9/5/2007 11:03'!
name
	^ name ifNil: [subject name]! !

!ClassEditor methodsFor: 'initialize-release' stamp: 'mtf 6/27/2008 10:05'!
on: aClass for: aSystemEditor
	^ self setSubject: aClass system: aSystemEditor! !

!ClassEditor methodsFor: 'debugging' stamp: 'mtf 9/20/2007 13:15'!
parserClass
	^ self class parserClass! !

!ClassEditor methodsFor: 'printing' stamp: 'cwp 10/23/2005 02:58'!
printOn: aStream
	aStream 
		nextPutAll: 'a ClassEditor on: ';
		nextPutAll: subject name! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 4/14/2006 19:23'!
product
	^ product ifNil: [product := self edRequiresBuild ifTrue: [self edBuild] ifFalse: [subject]]! !

!ClassEditor methodsFor: 'editing' stamp: 'gk 9/3/2007 23:44'!
rearrangeClassVarNames: associations
	"The associations are name -> pos and cover the subset
	of vars that should be moved. The old just fill in the holes."

	classVarNames := ClassEditor rearrangeVarNames: associations oldNames: classVarNames! !

!ClassEditor methodsFor: 'editing' stamp: 'gk 9/5/2007 00:22'!
rearrangeSharedPoolVarNames: associations
	"The associations are name -> pos and cover the subset
	of vars that should be moved. The old just fill in the holes.
	NOTE: This method uses identityIncludes:, otherwise it fails on empty Dictionaries."

	| newPools |
	newPools := Array new: sharedPools size.
	associations do: [:assoc |
		newPools at: assoc value put: (
			system edResolve: assoc key asSymbol
				ifAbsent: [self error: 'Can not resolve pool ', assoc key])].
	sharedPools do: [:oldPool |
		(newPools identityIncludes: oldPool) ifFalse: [
			newPools at: (newPools indexOf: nil) put: oldPool]].
	sharedPools := newPools! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:58'!
removeClassVarName: aString 
	classVarNames ifNil: [classVarNames := subject classVarNames].
	classVarNames := classVarNames copyWithout: aString! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 12/3/2005 20:24'!
removeFromSystem
	system edRemoveClassNamed: self name! !

!ClassEditor methodsFor: 'editing' stamp: 'gk 9/5/2007 00:10'!
removeSharedPool: aSharedPool
	"We use reject based on identity instead of copyWithout: here."

	sharedPools ifNil: [sharedPools := subject sharedPools].
	sharedPools := sharedPools reject: [:each | each == aSharedPool]! !

!ClassEditor methodsFor: 'editing' stamp: 'mtf 11/17/2007 23:58'!
rename: aString
	| oldName newName |
	(newName := aString asSymbol) = (oldName := self name)
		ifTrue: [^ self].
	(self system includesKey: newName)
		ifTrue: [^ self error: newName , ' already exists'].
	self setName: newName.
	self category: self category. "set explicitly to trigger reCategorize"
	self system renameClass: self from: oldName! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 11/4/2005 08:14'!
setName: aSymbol
	name := aSymbol! !

!ClassEditor methodsFor: 'initialize-release' stamp: 'mtf 6/20/2008 00:09'!
setSubject: aClass system: aSystemEditor 
	subject := aClass.
	system := aSystemEditor. 
	subject ifNil: 
			[superEditor := aSystemEditor at: #Object.
			name := #Annonymous.
			type := #normal.
			instVarNames := Array new.
			classVarNames := Array new.
			sharedPools := Array new.
			methods := MethodDictionaryEditor for: self.
			self class edInitializeForNewClass].
		decorators := ClassDecorator decoratorsFor: self.! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/2/2005 23:11'!
sharedPoolsString
	^ String streamContents: [ :stream |
		self sharedPools 
			do: [ :each |
				stream nextPutAll: (system 
								keyAtIdentityValue: each 
								ifAbsent: ['private'])]
			separatedBy: [stream space]]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 18:57'!
sharedPools
	^ sharedPools ifNil: [subject sharedPools]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 9/18/2008 12:08'!
subclassesOrEditors
"Answer all of my subclasses, as either classes or editors, whichever is easier"

	^ system edSubclassesOrEditorsOf: self! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 11/23/2005 21:51'!
subclasses
	^ system edSubclassesOf: self! !

!ClassEditor methodsFor: 'editing' stamp: 'kph 5/26/2007 05:58'!
subclass: aSymbol instanceVariableNames: instString classVariableNames: classString poolDictionaries: poolString category: categoryString 
	^ ((system 
		at: aSymbol
		ifAbsent: [ ClassEditor on: nil for: system])
			edName: aSymbol
			superclassEditor: self
			type: #normal
			instVarString: instString
			classVarString: classString
			poolImports: poolString
			category: categoryString) edRegisterEditor! !

!ClassEditor methodsFor: 'accessing' stamp: 'cwp 10/23/2005 01:54'!
subject
	^ subject! !

!ClassEditor methodsFor: 'editing' stamp: 'mtf 9/20/2008 07:20'!
superclass: aClassOrEditor 
	super superclass: aClassOrEditor.
	self class superclass: aClassOrEditor class! !

!ClassEditor methodsFor: 'accessing' stamp: 'cwp 10/30/2005 12:33'!
system
	^ system! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 9/13/2007 19:53'!
theMetaClass
	^ self class! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 9/13/2007 19:53'!
theNonMetaClass
	^ self! !

ClassEditor subclass: #RootClassEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!RootClassEditor commentStamp: 'mtf 7/29/2008 08:37' prior: 0!
My instances represent ClassEditors on the concept of "ProtoObject's superclass", the primitive object from which every class, without exception, inherits. I exist so that queries that involve the superclass do not have to be special-cased in case the superclass really is nil.!

!RootClassEditor class methodsFor: 'instance creation' stamp: 'mtf 7/29/2008 08:32'!
canEdit: anObject for: anEditor
	^ false! !

!RootClassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:33'!
classMetaclassEditor
	^ OldRootMetaclassEditor! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 7/28/2008 14:04'!
allInstVarNames
	^ #()! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:58'!
allSuperclasses
	^ OrderedCollection new! !

!RootClassEditor methodsFor: 'initialize-release' stamp: 'mtf 7/28/2008 14:06'!
edRegisterEditor
"Do nothing"! !

!RootClassEditor methodsFor: 'building' stamp: 'mtf 7/28/2008 13:55'!
edRequiresBuild
	^ false! !

!RootClassEditor methodsFor: 'building' stamp: 'mtf 7/28/2008 14:02'!
edRequiresRecompile
	^ false! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 7/28/2008 14:03'!
instSize
	^ 0! !

!RootClassEditor methodsFor: 'building' stamp: 'mtf 7/28/2008 14:03'!
product
	^ nil! !

!RootClassEditor methodsFor: 'accessing' stamp: 'mtf 7/28/2008 14:03'!
subject
	^ nil! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 7/28/2008 14:03'!
typeOfClass
	^ #normal! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:54'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
"Return the source code at selector, deferring to superclass if necessary"

	^ aBlock value! !

!ClassEditor class methodsFor: 'initialize-release' stamp: 'cwp 11/20/2005 18:36'!
initialize
	| strings |
	strings := #('self' 'super' 'true' 'false' 'nil' 'thisContext').
	ReservedNames := strings, strings collect: [:ea | ea asSymbol].! !

ClassDescriptionEditor subclass: #MetaclassEditor
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!MetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:34'!
classClassEditor
"Answers the class of my instance-side editor"

	^ ClassEditor! !

!MetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:34'!
new
	| instance |
	instance := self basicNew.
	instance
		superclass: self classClassEditor
		methodDictionary: MethodDictionary new
		format: self classClassEditor format.
	^ instance! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
acceptsLoggingOfCompilation

	^ self instanceSide acceptsLoggingOfCompilation! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
addInstVarName: aString 
	instVarNames ifNil: [instVarNames := self subject instVarNames].
	instVarNames := instVarNames copyWith: aString! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
allInstVarNames
	 "specialized in order to enable debugger to show as self"
	
	self isDebuggingAsEditor ifTrue: [^ super allInstVarNames].
	^ self edSuperclass
		ifNil: [self instVarNames copy]
		ifNotNil: [self edSuperclass allInstVarNames, self instVarNames]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
browserDefinition: style
	^ self definition! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
classPool
	^ editor classPool! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
classSide
	^self theMetaClass! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
comment
	^ self instanceSide comment! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
compiledMethodAt: aSelector
	^ self subjectDo: [:subject | subject compiledMethodAt: aSelector]! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
compile: source classified: cat notifying: anObject
	^ self 
		compile: source
		classified: cat
		withStamp: Utilities changeStamp
		notifying: anObject
		logSource: self acceptsLoggingOfCompilation ! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
compile: source classified: category withStamp: aString notifying: anObject logSource: aBoolean
	| cm |
	cm := MethodEditor
			source: source
			classified: category
			stamp: aString
			notifying: anObject
			logging: aBoolean.
	self methods add: cm.! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
compile: source classified: cat withStamp: aString notifying: anObject
	^ self 
		compile: source
		classified: cat
		withStamp: aString
		notifying: anObject
		logSource: self acceptsLoggingOfCompilation ! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
compile: source classified: cat
	^ self 
		compile: source
		classified: cat
		withStamp: Utilities changeStamp
		notifying: nil
		logSource: self acceptsLoggingOfCompilation ! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
definitionST80
	^ String streamContents: 
		[:strm |
		strm print: self;
			crtab;
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
definitionST80: isST80
	^ isST80 ifTrue: [self definitionST80] ifFalse: [self definition]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
definition
	^ self definitionST80! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 12:04'!
edBuild
	| result class |
	class := self subject ifNil: [Metaclass] ifNotNilDo: [:subject | subject class].
	result := class basicNew.
	"Create a temporary MethodDictionary to catch code written by SyntaxError dialogs. MethodDictionaryEditor will overwrite this. See MethodEditor>>compileFor:"
	result
		superclass: self edSuperclass
		methodDictionary: MethodDictionary new
		format: self format;		
		organization: self organization build.
	result setInstVarNames: self instVarNames.
	^ result! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edClassFormat
	^ ClassFormat
		size: self instSize
		type: self typeOfClass
		index: self indexIfCompact! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edInitializeForNewClass
	superEditor := self system edEditorFor: Object class.
	instVarNames := Array new.
	methods := MethodDictionaryEditor for: self! !

!MetaclassEditor methodsFor: 'testing' stamp: 'mtf 8/13/2008 00:34'!
edIsEditor
	^ true! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
edMethodAt: aSelector ifAbsent: aBlock
	^ self methods at: aSelector ifAbsent: aBlock! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
edMethodAt: aSelector
	^ self methods at: aSelector! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edMethodsDo: aBlock
	methods ifNotNil: [methods do: aBlock]! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edRequiresBuild
	^ (superEditor ~~ nil) | (instVarNames ~~ nil) | (methods ~~ nil)! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edRequiresRecompile
	^ instVarNames ~~ nil! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edRequiresSubclassRebuild
	^ self subject
		ifNotNilDo: [:subject | subject format ~= self format]
		ifNil: [false]! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edSuperclass
	^ superEditor
		ifNil: [self subject superclass]
		ifNotNil: [superEditor product]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
environment
	^ self system! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
format
"Override the implemenation in Behavior. That will return the format of ClassEditor, but we want the format of the subject's meta class"

	^ self edClassFormat bits! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
hasComment
	^ self instanceSide hasComment! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
includesSelector: aSelector
	^ (methods ifNil: [self subject] ifNotNil: [methods])
		includesSelector: aSelector! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
indexIfCompact
	self subject ifNotNil: [^ self subject indexIfCompact].
	^ 0! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
instanceSide
	^ self theNonMetaClass! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
instanceVariableNames: instString
	instVarNames := Scanner new scanFieldNames: instString! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
instanceVariablesString
	^ String streamContents: 
		[:stream | 
		self allInstVarNames
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream nextPut: $ ]]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
instSize
	"Override the implemenation in Behavior, so that when inspecting the
	ClassEditor, it will look like a regular Class"

	self isDebuggingAsEditor ifTrue: [^ super instSize].
	^ self instVarNames size + (self edSuperclass
		ifNil: [0] ifNotNilDo: [:ed | ed instSize])! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
instVarNames
	"Override the implemenation in Behavior. That will return the inst vars of 
	ClassEditor, but we want the inst vars of the subject's class."

	self isDebuggingAsEditor ifTrue: [^ super instVarNames].
	^ instVarNames ifNil: [self subject instVarNames]! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 9/18/2008 10:47'!
isDebuggingAsEditor
	"To be transparent enough to fool most existing software,
	MetaclassEditors must answer their subject's instance variables when
	sent the message allInstVarNames. However, this message is also used by
	the debugger, inspector, and explorer, when inspecting ClassEditors, and
	an inspector should see the real instance variables of ClassEditor rather
	than those of the MetaclassEditor's subject. So there is a conflict of
	interest already.
	
	But it gets worse. If you do open an inspector on an object that is
	reporting wrong information about it's instance variables, Bad things
	happen. Specifically, the inspector calls instVarAt: on an instance
	variable that does not exist, leading to an error every time the display
	is updated. Less severe, instVarAt: will answer nonsensical. 
	
	So I took the instance variable proxying behavior all the way so that it
	successfully fools the debugger, inspector, and explorer into seeing
	proxied instance variables (see the senders af this message if you want
	to know how). But there is still the original problem that you, the
	programmer want to see the real state of ClassEditor. That is where this
	method comes in.
	
	If this method returns false, everybody sees the proxied instance
	variables. This is the correct behavior. If this method returns false,
	everyone sees the true instance variables of ClassEditor. This is
	incorrect behavior, and breaks the Unit tests, but it is invaluable if
	you are debugging ClassEditor"

	^ self environment debug! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
isMeta
	^ true! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
isUniClass
	"The debugger sometimes sends this message"
	^ false! !

!MetaclassEditor methodsFor: 'metaclass' stamp: 'mtf 8/13/2008 00:34'!
lookupSelector: aSelector
	
	"Override the implementation in Behavior to make ClassEditor
	debuggable. The debugger calls this during simulated execution,
	and since we override #includesSelector and #superclass to 
	reflect on the subject, we get incorrect method dispatch."

	| lookupClass dict |
	lookupClass := self.
	[lookupClass == nil] whileFalse: 
		[dict := lookupClass instVarAt: 2.
		(dict includesKey: aSelector)
			ifTrue: [^ dict at: aSelector].
		lookupClass := lookupClass instVarAt: 1].
	^ nil! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
methodDictionary
	| methodEditor |
	(methods isNil and: [self edRequiresRecompile not]) ifTrue: [^ self subject methodDictionary].
	methodEditor := methods ifNil: [MethodDictionaryEditor for: self].
	^ methodEditor buildFor: self! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'mtf 8/13/2008 00:34'!
methods
	^ methods ifNil: [methods := MethodDictionaryEditor for: self]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
name
	^ editor name, ' class'! !

!MetaclassEditor methodsFor: 'metaclass' stamp: 'mtf 8/13/2008 00:34'!
new
	^ editor := self basicNew! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
organization
	^ organization ifNil: [organization := OrganizationEditor for: self]! !

!MetaclassEditor methodsFor: 'printing' stamp: 'mtf 8/13/2008 00:34'!
printOn: aStream
	aStream
		nextPutAll: 'a MetaclassEditor on: ';
		print: editor subject ! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
product
	^ editor product class! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
rearrangeInstVarNames: associations
	"The associations are name -> pos and cover the subset
	of vars that should be moved. The old just fill in the holes."

	instVarNames := ClassEditor rearrangeVarNames: associations oldNames: instVarNames! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
removeInstVarName: aString 
	instVarNames ifNil: [instVarNames := self subject instVarNames].
	instVarNames := instVarNames copyWithout: aString! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
removeSelector: aSymbol 
	methods ifNil: [methods := MethodDictionaryEditor for: self].
	methods remove: aSymbol.! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
sourceCodeAt: aSelector
	^ self subjectDo: [:subject | subject sourceCodeAt: aSelector]! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'mtf 8/13/2008 00:34'!
subjectDo: aBlock
	^ self subject
		ifNotNilDo: [:subject | aBlock value: subject]
		ifNil: [self error: 'key not found']! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'mtf 8/13/2008 00:34'!
subject
	^ editor subject ifNotNilDo: [:subject | subject class]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
superclass
	| subjectSuperclass |
	superEditor ifNotNil: [^ superEditor].
	subjectSuperclass := self subject
		ifNil: [Object]
		ifNotNil: [self subject superclass].
	^ self system edEditorFor: subjectSuperclass! !

!MetaclassEditor methodsFor: 'editing' stamp: 'mtf 8/13/2008 00:34'!
superclass: aClass 
	superEditor := self system edEditorFor: aClass! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'mtf 8/13/2008 00:34'!
system
	^ editor system! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
theMetaClass
	^ self! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
theNonMetaClass
	^ editor! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
typeOfClass
	^ #normal! !

MetaclassEditor subclass: #RootMetaclassEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!RootMetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:35'!
canEdit: anObject for: anEditor
	^ false! !

!RootMetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:35'!
classClassEditor
	^ RootClassEditor! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:35'!
allInstVarNames
	^ #()! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:35'!
allSuperclasses
	^ OrderedCollection new! !

!RootMetaclassEditor methodsFor: 'initialize-release' stamp: 'mtf 8/13/2008 00:35'!
edRegisterEditor
"Do nothing"! !

!RootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:35'!
edRequiresBuild
	^ false! !

!RootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:35'!
edRequiresRecompile
	^ false! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:35'!
instSize
	^ 0! !

!RootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:35'!
product
	^ nil! !

!RootMetaclassEditor methodsFor: 'accessing' stamp: 'mtf 8/13/2008 00:35'!
subject
	^ nil! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:35'!
typeOfClass
	^ #normal! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:35'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
"Return the source code at selector, deferring to superclass if necessary"

	^ aBlock value! !

AbstractEditor subclass: #DictionaryEditor
	instanceVariableNames: 'subject additions removals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!DictionaryEditor methodsFor: 'testing' stamp: 'mtf 10/6/2008 13:43'!
isChanged
	^ additions notEmpty or: [removals notEmpty]! !

DictionaryEditor subclass: #MethodDictionaryEditor
	instanceVariableNames: 'classEditor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!MethodDictionaryEditor class methodsFor: 'instance creation' stamp: 'cwp 11/3/2005 07:30'!
empty
	^ self on: MethodDictionary new! !

!MethodDictionaryEditor class methodsFor: 'instance creation' stamp: 'mtf 9/11/2007 23:31'!
for: aClassEditor 
	^ self new setClassEditor: aClassEditor! !

!MethodDictionaryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 23:02'!
additions
	^ additions! !

!MethodDictionaryEditor methodsFor: 'editing' stamp: 'mtf 9/11/2007 23:58'!
add: aModifiedMethod 
	^ additions at: aModifiedMethod selector put: aModifiedMethod! !

!MethodDictionaryEditor methodsFor: 'reflecting' stamp: 'mtf 9/20/2007 14:11'!
at: aSymbol ifAbsent: absentBlock
	(removals includes: aSymbol) ifTrue: [^ absentBlock value].
	additions at: aSymbol ifPresent: [:editor | ^ editor].
	self subject at: aSymbol ifPresent: [:aCompiledMethod | ^ self add: (MethodEditor
		source: (classEditor subject sourceCodeAt: aSymbol)
		classified: (classEditor organization subject categoryOfElement: aSymbol)
		stamp: aCompiledMethod timeStamp
		notifying: nil
		logging: true )].
	^ absentBlock value! !

!MethodDictionaryEditor methodsFor: 'reflecting' stamp: 'mtf 9/12/2007 00:03'!
at: aSymbol
	^ self at: aSymbol ifAbsent: [self error: 'Method not found']! !

!MethodDictionaryEditor methodsFor: 'editing' stamp: 'mtf 10/6/2008 13:57'!
buildFor: aClassEditor
	| result old |
self halt.
	result := MethodDictionary new.
	self subject keysAndValuesDo:
		[:selector :cm |
		(self selectorIsModified: selector)
			ifFalse: [result at: selector put: (self recompile: selector from: cm for: aClassEditor)]].
	additions keysAndValuesDo: 
		[:selector :mm | 
		old := self subject at: selector ifAbsent: [nil]. 
		result at: selector put: (mm compileFrom: old for: aClassEditor)].
	^ result
! !

!MethodDictionaryEditor methodsFor: 'building' stamp: 'cwp 1/14/2006 19:02'!
do: aBlock
	additions do: aBlock! !

!MethodDictionaryEditor methodsFor: 'building' stamp: 'mtf 9/12/2007 10:26'!
edPrepareMigration: txn for: aClassEditor
	txn addMigrator: (ObjectMigrator 
						origin: self subject 
						destination: (self buildFor: aClassEditor)).
	(additions isEmpty and: [removals isEmpty]) ifFalse: [aClassEditor organization].! !

!MethodDictionaryEditor methodsFor: 'building' stamp: 'cwp 2/1/2006 21:42'!
edRequiresBuild
	^ true! !

!MethodDictionaryEditor methodsFor: 'testing' stamp: 'mtf 9/11/2007 23:27'!
includesSelector: aSymbol 
	(additions includesKey: aSymbol) ifTrue: [^ true].
	(removals includes: aSymbol) ifTrue: [^false].
	(self subject includesKey: aSymbol) ifTrue: [^ true].
	^ false! !

!MethodDictionaryEditor methodsFor: 'building' stamp: 'mtf 8/12/2008 15:15'!
recompile: aSelector from: aCompiledMethod for: aClassEditor
	| trailer node |
	aClassEditor edRequiresRecompile ifFalse: [^ aCompiledMethod].
	trailer := aCompiledMethod trailer.
	node := aClassEditor compilerClass new
			compile: (aCompiledMethod 
						getSourceFor: aSelector 
						in: aClassEditor)
			in: aClassEditor product
			notifying: nil
			ifFail: [self error: 'Compilation failed'].
	node selector == aSelector ifFalse: [self error: 'selector changed!!'].
	^ node generate: trailer! !

!MethodDictionaryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 23:02'!
removals
	^ removals! !

!MethodDictionaryEditor methodsFor: 'editing' stamp: 'cwp 10/25/2005 23:08'!
remove: aSymbol 
	additions removeKey: aSymbol ifAbsent: [].
	removals add: aSymbol! !

!MethodDictionaryEditor methodsFor: 'testing' stamp: 'cwp 10/26/2005 08:44'!
selectorIsModified: aSelector
	^ (additions includesKey: aSelector) or: [removals includes: aSelector]! !

!MethodDictionaryEditor methodsFor: 'initialization' stamp: 'mtf 9/11/2007 23:29'!
setClassEditor: aClassEditor
	classEditor := aClassEditor.
	additions := IdentityDictionary new.
	removals := IdentitySet new.! !

!MethodDictionaryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 23:54'!
subject
	^ subject ifNil: [classEditor subject 
		ifNil: [subject := MethodDictionary new]
		ifNotNilDo: [:aClass | subject := aClass methodDictionary]]! !

DictionaryEditor subclass: #SystemEditor
	instanceVariableNames: 'organization showProgress debug'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!SystemEditor class methodsFor: 'instance creation' stamp: 'cwp 10/28/2005 08:51'!
new
	^ self on: Smalltalk! !

!SystemEditor class methodsFor: 'instance creation' stamp: 'cwp 10/28/2005 08:51'!
on: anEnvironment
	^ self basicNew setSubject: anEnvironment! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:50'!
associationAt: aSymbol ifAbsent: aBlock
 
	self at: aSymbol ifAbsent: [^ aBlock value]. "first create any editors necessary"
	^ additions associationAt: aSymbol
		ifAbsent: [subject associationAt: aSymbol]! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 6/26/2008 11:18'!
associationAt: aSymbol 
	^ self 
		associationAt: aSymbol 
		ifAbsent: [self error: 'key not found']! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:49'!
at: aSymbol ifAbsent: aBlock
	| newEditor |
	(removals includes: aSymbol) ifTrue: [^ aBlock value].
	additions at: aSymbol ifPresent: [:editor | ^ editor].
	subject at: aSymbol ifPresent: [:obj |
		newEditor := AbstractEditor on: obj for: self ifNotHandled: [^ obj].
		^ newEditor edRegisterEditor].
	^ aBlock value! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 6/12/2008 23:20'!
at: key ifPresent: aBlock
	"Lookup the given key in the receiver. If it is present, answer the value of evaluating the given block with the value associated with the key. Otherwise, answer nil."

	| v |
	v := self at: key ifAbsent: [^ nil].
	^ aBlock value: v
! !

!SystemEditor methodsFor: 'reflecting' stamp: 'cwp 4/14/2006 16:24'!
at: aSymbol 
	^ self 
		at: aSymbol 
		ifAbsent: [self error: 'key not found']! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 6/26/2008 11:21'!
bindingOf: aSymbol
 
	^ self associationAt: aSymbol ifAbsent: [ self error: 'could  not bind ', aSymbol ]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/8/2008 23:03'!
commitWithProgress

	showProgress := true.
	self commit.
	showProgress := false.! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 10/7/2008 10:56'!
commit
	| migration export |
	showProgress ifNil: [showProgress := false].
	self validate.

	self edExpandEditors.

	migration := MigrationTransaction new.
	self edPrepareMigration: migration.

	export := ClassExporter on: subject.
	self edPrepareExport: export.

	showProgress ifTrue: [(1 to: 2) do: [:ea | ] displayingProgress: 'Committing...'].
	[migration commit.
	export commit.
	self edCommitRemovals.
	self edRecategorize.] valueUnpreemptively.
	
	^ true! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 9/18/2008 10:48'!
debug
"see MetaclassEditor>>isDebuggingAsEditor"

	^ debug ifNil: [false]! !

!SystemEditor methodsFor: 'debugging' stamp: 'mtf 9/18/2008 10:44'!
debug: aBoolean
"see MetaclassEditor>>isDebuggingAsEditor"

	debug := aBoolean! !

!SystemEditor methodsFor: 'building' stamp: 'cwp 11/6/2005 23:24'!
edClassAt: aSymbol ifAbsent: aBlock
	^ subject at: aSymbol ifAbsent: aBlock! !

!SystemEditor methodsFor: 'building' stamp: 'cwp 12/3/2005 21:26'!
edCommitRemovals
	removals do: [:ea | self edRemove: ea]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 6/11/2008 20:49'!
edEditorFor: aClass
	| name editor |
	aClass edIsEditor ifTrue: [^ aClass].
	name := aClass theNonMetaClass name.
	editor := self at: name.
	^ aClass isMeta ifFalse: [editor] ifTrue: [editor class].! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:50'!
edExpandEditors
	| queue editor |
	queue := additions values asOrderedCollection.
	[queue isEmpty] whileFalse: 
		[editor := queue removeFirst.
		editor subject ifNotNil: [
			queue addAll:  editor edExpand]]
			
			! !

!SystemEditor methodsFor: 'testing' stamp: 'cwp 11/20/2005 16:29'!
edIsEditor
	^ true! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 8/14/2008 13:49'!
editors
	^ additions! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:49'!
edPrepareExport: exporter

	showProgress
		ifFalse:	[additions do: [:ea | ea edPrepareExport: exporter]]
		ifTrue:	[additions do: [:ea | ea edPrepareExport: exporter] displayingProgress: 'Preparing Classes...']! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:48'!
edPrepareMigration: txn

	showProgress
		ifFalse:	[additions do: [:ea | ea edPrepareMigration: txn]]
		ifTrue:	[additions do: [:ea | ea edPrepareMigration: txn] displayingProgress: 'Preparing Transaction...']! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:48'!
edRecategorize
	additions do: 
		[:ea | 
		ea edCategory ifNotNilDo:
			[:cat | 
			SystemOrganization 
				classify: ea name
				under: cat]]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:50'!
edRegisterEditor: anEditor 
	additions at: anEditor name put: anEditor! !

!SystemEditor methodsFor: 'editing' stamp: 'mtf 8/14/2008 13:50'!
edRemoveClassNamed: aSymbol
	additions removeKey: aSymbol ifAbsent: [].
	removals add: aSymbol! !

!SystemEditor methodsFor: 'building' stamp: 'cwp 12/3/2005 21:27'!
edRemove: aSymbol
	| object |
	object := subject at: aSymbol ifAbsent: [^ self].
	object isBehavior
		ifTrue: [object removeFromSystem]
		ifFalse: [subject removeKey: aSymbol]! !

!SystemEditor methodsFor: 'building' stamp: 'gk 9/5/2007 00:02'!
edResolve: aSymbol ifAbsent: aBlock
	^ subject at: aSymbol ifAbsent: aBlock! !

!SystemEditor methodsFor: 'building' stamp: 'cwp 11/3/2005 07:53'!
edResolve: aSymbol
	^ subject at: aSymbol! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 9/18/2008 12:06'!
edSubclassesOf: anEditor
"Answers all subclasses of the given ClassEditor, as editors"

	^ (self edSubclassesOrEditorsOf: anEditor) collect: [:ea | self edEditorFor: ea]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 9/19/2008 11:30'!
edSubclassesOrEditorsOf: anEditor
"Answer all subclasses of the given ClassEditor, as either an editor (if it has changed), or as a class (if it has not changed). Does not add any items to my additions list"

	| subclasses subeditors |
	subclasses := anEditor subject
		ifNil: [Set new]
		ifNotNilDo: [ :class | class subclasses asSet].
	subeditors := additions select: [ :ea |
		subclasses remove: ea subject ifAbsent: [].
		ea superclassOrEditor == anEditor].
	removals do: [ :removedKey | subclasses remove: (subject at: removedKey) ifAbsent: []].
	^ subclasses asArray, subeditors asArray! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:49'!
hasBindingThatBeginsWith: aString
	"Answer true if the receiver has a key that begins with aString, false otherwise"
	
	additions keysDo:[:each | 
		((each beginsWith: aString)
			and: [(removals includes: each) not]) ifTrue:[^true]].
	subject keysDo:[:each | 
		((each beginsWith: aString)
			and: [(removals includes: each) not]) ifTrue:[^true]].
	^false! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:50'!
includesKey: aSymbol 
	(removals includes: aSymbol) ifTrue: [^ false].
	(additions includesKey: aSymbol) ifTrue:  [^ true].
	subject ifNil: [^ false].
	(subject includesKey: aSymbol) ifTrue: [^ true].
	^ false! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:46'!
keyAtIdentityValue: anObject ifAbsent: aBlock
	^ additions 
		keyAtIdentityValue: anObject
		ifAbsent: [subject 
					keyAtIdentityValue: anObject 
					ifAbsent: aBlock]! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 22:47'!
organization
	^ organization ifNil: [organization := SystemOrganizationEditor for: self]! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 20:51'!
removals
	^ removals! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 9/4/2007 15:52'!
renameClass: aClassEditor from: aSymbol 
	self edRemoveClassNamed: aSymbol.
	aClassEditor edRegisterEditor! !

!SystemEditor methodsFor: 'initialize-release' stamp: 'mtf 8/14/2008 13:44'!
setSubject: anEnvironment 
	subject := anEnvironment.
	additions := IdentityDictionary new.
	removals := Set new.! !

!SystemEditor methodsFor: 'accessing' stamp: 'cwp 10/28/2005 08:49'!
subject
	^ subject! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:44'!
validate
	additions do: [:ea | ea validate]! !

Object subclass: #ClassDecorator
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ClassDecorator commentStamp: 'mtf 6/19/2008 23:29' prior: 0!
An abstract superclass for extensions to ClassEditor and MetaclassEditor!

!ClassDecorator class methodsFor: 'as yet unclassified' stamp: 'mtf 6/28/2008 13:36'!
decoratorsFor: anEditor
	^ self allSubclasses
		collect: [:class | class for: anEditor]
		thenSelect: [:ea | ea notNil]! !

!ClassDecorator class methodsFor: 'as yet unclassified' stamp: 'mtf 6/20/2008 00:11'!
for: anEditor
	"Answer a decorator appropriate for anEditor, or nil if not applicable"
	^ self new for: anEditor! !

!ClassDecorator methodsFor: 'initialize-release' stamp: 'mtf 6/20/2008 00:10'!
for: anEditor
	editor := anEditor! !

Object subclass: #ClassExporter
	instanceVariableNames: 'environment classes oldClasses newClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ClassExporter class methodsFor: 'instance creation' stamp: 'cwp 11/5/2005 17:11'!
on: anEnvironment
	^ self new setEnvironment: anEnvironment! !

!ClassExporter methodsFor: 'public' stamp: 'cwp 11/5/2005 17:11'!
addClass: aClass
	classes add: aClass! !

!ClassExporter methodsFor: 'public' stamp: 'cwp 11/5/2005 18:46'!
commit
	[classes do: [:ea | self export: ea].
	self forwardIdentities]
		valueUnpreemptively
! !

!ClassExporter methodsFor: 'public' stamp: 'cwp 11/5/2005 18:43'!
environment
	^ environment! !

!ClassExporter methodsFor: 'private' stamp: 'cwp 11/5/2005 18:18'!
export: aClass 
	aClass superclass addSubclass: aClass.
	(environment includesKey: aClass name)
		ifTrue: [self replace: aClass]
		ifFalse: [environment at: aClass name put: aClass]! !

!ClassExporter methodsFor: 'private' stamp: 'cwp 11/7/2005 20:25'!
forwardIdentities
	oldClasses := oldClasses asArray.
	newClasses := newClasses asArray.
	oldClasses  elementsForwardIdentityTo: newClasses.
	oldClasses := nil.
	newClasses := nil.! !

!ClassExporter methodsFor: 'private' stamp: 'cwp 11/7/2005 20:27'!
replace: aClass
	oldClasses add: (environment at: aClass name).
	newClasses add: aClass! !

!ClassExporter methodsFor: 'initialize-release' stamp: 'cwp 11/5/2005 18:36'!
setEnvironment: anEnvironment 
	environment := anEnvironment.
	classes := OrderedCollection new.
	oldClasses := OrderedCollection new.
	newClasses := OrderedCollection new.! !

Object subclass: #ClassFormat
	instanceVariableNames: 'instSize instSpec cClassIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ClassFormat commentStamp: 'mtf 7/7/2008 23:18' prior: 0!
I am an expert on the way the Squeak VM encodes class formats. My instance variable, 'bits', is identical in format to the 'format' instance variable of Behavior. I have methods for interpreting those bits. The class format integer is 18 bits in length. The instSize field is split into two parts for backwards compatibility.

<2 bits=instSize//64> <5 bits=cClass> <4 bits=instSpec> <6 bits=instSize\\64> <1 bit=0>

- instSize: the number of named instance variables
- cClass: the index into the compact class table
- instSpec: values indicating how instances are organized
	0 - no state
	1 - named instance variables only
	2 - variable number of object pointers
	3 - named instance variables, variable number of object pointers
	4 - variable number of weak object pointers
	6 - variable number of words
	8 - variable number of bytes
	12 - compiled method format: variable number of object pointers, variable number of bytes!

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
bytesSpec
	^ 8! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:14'!
bytes
	^ self size: 0 spec: self bytesSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'mtf 7/3/2008 12:11'!
compiledMethodSpec
	^ 12! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:41'!
fromBits: bits
	^ self
		size: (((bits bitShift: -10) bitAnd: 192) + ((bits bitShift: -1) bitAnd: 63) - 1)
		spec: ((bits bitShift: -7) bitAnd: 15)
		index: ((bits bitShift: -11) bitAnd: 31)! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:57'!
indexedSpec
	^ 2! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 18:08'!
indexedSpec: size
	^ size = 0 ifTrue: [self indexedSpec] ifFalse: [self namedAndIndexedSpec]! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:12'!
indexed
	^ self indexed: 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:03'!
indexed: size
	^ self 
		size: size
		spec: (self indexedSpec: size)
		index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:57'!
namedAndIndexedSpec
	^ 3! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:55'!
namedSpec
	^ 1! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:59'!
named
	^ self named: 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:06'!
named: size
	^ self
		size: size
		spec: self namedSpec
		index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:55'!
noStateSpec
	^ 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:16'!
noState
	^ self size: 0 spec: self noStateSpec index: 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:41'!
size: instSize spec: instSpec index: cClassIndex
	^ self new setSize: instSize spec: instSpec index: cClassIndex! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'mtf 7/8/2008 12:22'!
size: instSize type: aSymbol index: cClassIndex
	^ self size: instSize spec: (self specFromType: aSymbol size: instSize) index: cClassIndex! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'mtf 7/3/2008 12:10'!
specFromType: aSymbol size: size
	^ aSymbol caseOf: {
		[#normal] -> [size > 0 ifTrue: [self namedSpec] ifFalse: [self noStateSpec]].
		[#variable] -> [self indexedSpec: size].
		[#bytes] -> [self bytesSpec].
		[#words] -> [self wordsSpec].
		[#weak] -> [self weakSpec].
		[#compiledMethod] -> [self compiledMethodSpec].
	} otherwise: [self error: 'Unknown class type: ', aSymbol]
	! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
weakSpec
	^ 4! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:18'!
weak
	^ self size: 0 spec: self weakSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
wordsSpec
	^ 6! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:17'!
words
	^ self size: 0 spec: self wordsSpec index: 0! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
bits
	"Based on ClassBuilder>>format:variable:words:pointers:weak - ar 7/11/1999 06:39 "

	| bits |
	bits := (instSize+1) // 64. 	
	bits := (bits bitShift: 5) + cClassIndex.
	bits := (bits bitShift: 4) + instSpec.
	bits := (bits bitShift: 6) + ((instSize+1)\\64).  "+1 since prim instSize field includes header"
	bits := (bits bitShift: 1). "This shift plus integer bit lets wordSize work like byteSize"
	^ bits! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:36'!
cClassIndex
	^ cClassIndex! !

!ClassFormat methodsFor: 'errors' stamp: 'cwp 11/23/2005 22:37'!
error: aString
	InvalidClassFormat signal: aString! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 9/26/2004 18:15'!
indexedSubclass: nInstVars
	| newSize |
	self isPointers ifFalse: [self error: 'Can''t make a pointer subclass.'].
	newSize := self instSize + nInstVars.
	newSize > 254 ifTrue: [self error: 'Too many instance variables'].
	
	^ self class indexed: newSize! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
instSize
	^ instSize! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
instSpec
	^ instSpec! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:47'!
isBits
	"Answer whether the receiver contains just bits (not pointers)."

	^ self instSpec >= 6! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isBytes
	"Answer whether the receiver has 8-bit instance variables."

	^ self instSpec >= 8! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isFixed
	"Answer whether the receiver does not have a variable (indexable) part."

	^self isVariable not! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isPointers
	"Answer whether the receiver contains just pointers (not bits)."

	^self isBits not! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isVariable
	"Answer whether the receiver has indexable variables."

	^ self instSpec >= 2! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isWeak
	"Answer whether the receiver has contains weak references."
	^ self instSpec = 4! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:49'!
isWords
	"Answer whether the receiver has 32-bit instance variables."

	^ self isBytes not! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 12/11/2005 10:40'!
namedSubclass: nInstVars 
	| newSize |
	newSize := self instSize + nInstVars.
	newSize > 254 ifTrue: [self error: 'Too many instance variables'].
	^ newSize = 0
		ifTrue: [self class noState]
		ifFalse: [self class named: newSize]! !

!ClassFormat methodsFor: 'initializing' stamp: 'cwp 9/26/2004 17:39'!
setSize: size spec: spec index: index
	instSize := size.
	instSpec := spec.
	cClassIndex := index.! !

!ClassFormat methodsFor: 'accessing' stamp: 'mtf 7/8/2008 12:07'!
typeOfClass
	"Answer a symbol uniquely describing the type of the receiver"
	self instSpec = self class compiledMethodSpec ifTrue:[^#compiledMethod]. "Very special!!"
	self isBytes ifTrue:[^#bytes].
	(self isWords and:[self isPointers not]) ifTrue:[^#words].
	self isWeak ifTrue:[^#weak].
	self isVariable ifTrue:[^#variable].
	^#normal.! !

!ClassFormat methodsFor: 'validating' stamp: 'mtf 7/8/2008 12:59'!
validateAgainstOld: oldFormat
"Raise an error if a class with my format cannot replace a class with oldFormat. Adapted from ClassBuilder >> validateSubclass:canKeepLayoutFrom:forSubclassFormat:"

	oldFormat ifNil: [^ self]. "This class is new, so anything works"
	(self isWeak and: [oldFormat isVariable]) ifTrue: [^ self].
	self typeOfClass ~~ oldFormat typeOfClass
		ifTrue: [self error: 'The new class format is incompatable with the old']! !

!ClassFormat methodsFor: 'validating' stamp: 'mtf 7/8/2008 12:21'!
validateAgainstSuper: superFormat
"Raise an error if a class with my format cannot inherit from a class with superFormat. Adapted from the various checks in the public ClassBuilder methods"

	self instSize < superFormat instSize ifTrue: [self error: 'A subclass lost instance variables'].
	superFormat isFixed ifTrue: [^ self]. "Any format can inherit from #normal"
	superFormat isVariable ~~ self isVariable ifTrue: [self error: 'Cannot derive a fixed class from a variable one'].
	superFormat isBits ~~ self isBits ifTrue: [self error: 'Bit and Pointer classes cannot inherit from one another'].
	superFormat isBytes ~~ self isBytes ifTrue: [self error: 'Byte and Word classes cannot inherit from one another'].
	! !

!ClassFormat methodsFor: 'validating' stamp: 'mtf 7/8/2008 12:19'!
validate
"Raise  an error if I am an invalid format"

	instSize > 254 ifTrue: [self error: 'Too many instance variables (', instSize printString,')'].
	(self isBits and: [instSize > 0]) ifTrue: [self error: 'Only pointer formats can have named variables'].
	! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 9/26/2004 18:28'!
withInstSize: aSmallInteger
	self isPointers ifFalse: [self error: 'Only pointer formats can have named variables'].
	^ self class size: aSmallInteger spec: self instSpec index: self cClassIndex! !

Object subclass: #InstanceMigrator
	instanceVariableNames: 'map instSize origin destination'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!InstanceMigrator commentStamp: 'mtf 7/14/2008 23:51' prior: 0!
My instances are responsible for finding instances of an old class and building them in a new format, to be swapped during MigrationTransaction>>commit!

!InstanceMigrator class methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2005 08:24'!
from: oldClass to: newClass
	^ self new setOrigin: oldClass destination: newClass! !

!InstanceMigrator methodsFor: 'initialize-release' stamp: 'cwp 11/2/2005 08:38'!
createMap
	| oldInstVarNames |
	oldInstVarNames := origin allInstVarNames.
	^ destination allInstVarNames 
			collect: [:instVarName | oldInstVarNames indexOf: instVarName].
! !

!InstanceMigrator methodsFor: 'accessing' stamp: 'cwp 4/15/2006 19:18'!
destination
	^ destination! !

!InstanceMigrator methodsFor: 'accessing' stamp: 'cwp 11/2/2005 08:50'!
instances
	^ origin allInstances asArray! !

!InstanceMigrator methodsFor: 'migrating' stamp: 'cwp 4/15/2006 22:27'!
migrateFrom: oStream to: dStream
	self instances do:
		[:ea |
		oStream nextPut: ea.
		dStream nextPut: (self migrate: ea)]! !

!InstanceMigrator methodsFor: 'migrating' stamp: 'cwp 3/19/2006 19:28'!
migrate: oldInstance 
	| newInstance |
	newInstance := destination edInstanceBasedOn: oldInstance.
	1 to: instSize do: 
		[:offset | 
		(map at: offset) > 0 ifTrue: 
			[newInstance 
				instVarAt: offset
				put: (oldInstance instVarAt: (map at: offset))]].
	destination isVariable ifTrue: 
		[1 to: oldInstance basicSize do: 
			[:offset | newInstance basicAt: offset put: (oldInstance basicAt: offset)]].
	^newInstance! !

!InstanceMigrator methodsFor: 'accessing' stamp: 'cwp 4/14/2006 16:31'!
origin
	^ origin! !

!InstanceMigrator methodsFor: 'initialize-release' stamp: 'cwp 11/2/2005 08:25'!
setOrigin: oldClass destination: newClass
	origin := oldClass.
	destination := newClass.
	map := self createMap.
	instSize := destination instSize! !

Object subclass: #MethodEditor
	instanceVariableNames: 'source category stamp requestor log repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!MethodEditor class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2005 21:36'!
source: source classified: catString stamp: stamp notifying: requestor logging: aBoolean
	^ self new
		setSource: source
		classified: catString
		stamp: stamp
		notifying: requestor
		log: aBoolean! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 1/14/2006 19:03'!
category
	^ category! !

!MethodEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 12:02'!
compileFor: aClassEditor
[
	| node method |
	node := aClassEditor compilerClass new
				compile: source
				in: aClassEditor product
				notifying: requestor
				ifFail: nil.
	node encoder requestor: self.
	method := node generate: #(0 0 0 0).
	(method respondsTo: #selector:) "set selector on 3.9 and above"
		ifTrue: [method selector: node selector].
	^ method
] on: SyntaxErrorNotification do: [:ex |
	"In 3.10, the class category needs to be set"
	ex instVarNamed: #category put: aClassEditor category.
	"Let the user fix and install the fixed code into the class's old or temporary MethodDictionary"
	ex outer.
	"Now fetch and use that code instead"
	source := aClassEditor product sourceCodeAt: self selector ifAbsent: [^ nil].
	^ aClassEditor product compiledMethodAt: self selector
]! !

!MethodEditor methodsFor: 'building' stamp: 'cwp 2/1/2006 20:19'!
compileFrom: aCompiledMethod for: aClassEditor
	| result remote |
	result := self compileFor: aClassEditor.
	^ log 
		ifFalse: [result]
		ifTrue: 
			[remote := self logCompilationFrom: aCompiledMethod for: aClassEditor.
			result setSourcePosition: remote position inFile: remote sourceFileNumber]
! !

!MethodEditor methodsFor: 'category browsing' stamp: 'mtf 9/29/2007 14:04'!
elementName
	"The name this editor will be classified under"
	^ self selector! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 3/3/2006 15:27'!
interactive
	^ false! !

!MethodEditor methodsFor: 'building' stamp: 'cwp 2/1/2006 20:05'!
logCompilationFrom: aCompiledMethod for: aClassEditor
	^ aCompiledMethod
		ifNil: [self repository 
				storeMethod: source
				forClass: aClassEditor
				category: category
				stamped: stamp]
		ifNotNil: [self repository
					storeMethod: source
					forClass: aClassEditor
					category: category
					stamped: stamp
					replacing: aCompiledMethod]! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 2/1/2006 20:20'!
repository
	^ repository ifNil: [repository := SourceRepository default]! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 1/31/2006 22:54'!
repository: anObject
	repository := anObject! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 10/25/2005 22:50'!
selector
	^ Parser new parseSelector: source! !

!MethodEditor methodsFor: 'initialize-release' stamp: 'cwp 10/25/2005 22:47'!
setSource: sourceString classified: catString stamp: timestamp notifying: anObject log: aBoolean
	source := sourceString.
	category := catString.
	stamp := timestamp.
	requestor := anObject.
	log := aBoolean ! !

!MethodEditor methodsFor: 'accessing' stamp: 'mtf 11/10/2007 01:07'!
timeStamp
	^ stamp! !

Object subclass: #MigrationTransaction
	instanceVariableNames: 'migrators'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!MigrationTransaction methodsFor: 'public' stamp: 'cwp 11/2/2005 08:36'!
addMigrator: aMigrator
	migrators add: aMigrator! !

!MigrationTransaction methodsFor: 'public' stamp: 'cwp 4/15/2006 22:34'!
commit
	"The mass become: is a primitive, but we execute this method with preemption
	disabled because we don't want another process creating new instances while
	we're busy converting the existing instances."

	| old new |
	[
		old := WriteStream on: (Array new: migrators size).
		new := WriteStream on: (Array new: migrators size).
		migrators do: [:ea | ea migrateFrom: old to: new].
		old contents elementsExchangeIdentityWith: new contents.
	] valueUnpreemptively.! !

!MigrationTransaction methodsFor: 'accessing' stamp: 'cwp 4/15/2006 19:25'!
destinationFor: anObject
	migrators do: [:ea | ea origin = anObject ifTrue: [^ ea destination]].
	^ nil! !

!MigrationTransaction methodsFor: 'initialize-release' stamp: 'cwp 4/15/2006 23:42'!
initialize
	migrators := OrderedCollection new.! !

!MigrationTransaction methodsFor: 'public' stamp: 'cwp 4/14/2006 18:46'!
isEmpty
	^ migrators isEmpty! !

!MigrationTransaction methodsFor: 'accessing' stamp: 'cwp 4/14/2006 16:31'!
migrators
	^ migrators! !

Object subclass: #ObjectMigrator
	instanceVariableNames: 'origin destination'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ObjectMigrator commentStamp: 'mtf 7/14/2008 11:59' prior: 0!
My instances store an old and new version of a single object that will be swapped during MigrationTransaction>>commit!

!ObjectMigrator class methodsFor: 'instance creation' stamp: 'cwp 4/15/2006 00:35'!
origin: origin destination: destination 
	^ self new setOrigin: origin destination: destination! !

!ObjectMigrator methodsFor: 'accessing' stamp: 'cwp 4/15/2006 19:19'!
destination
	^ destination! !

!ObjectMigrator methodsFor: 'migrating' stamp: 'cwp 4/15/2006 22:36'!
migrateFrom: oStream to: dStream
	oStream nextPut: origin.
	dStream nextPut: destination.! !

!ObjectMigrator methodsFor: 'accessing' stamp: 'cwp 4/15/2006 00:38'!
origin
	^ origin! !

!ObjectMigrator methodsFor: 'initialize-release' stamp: 'cwp 4/15/2006 00:36'!
setOrigin: anObject1 destination: anObject2 
	origin := anObject1.
	destination := anObject2! !

!Object methodsFor: '*systemeditor-testing' stamp: 'cwp 3/3/2006 14:46'!
edIsEditor
	^ false! !

Object subclass: #SourceRepository
	instanceVariableNames: 'files'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!SourceRepository commentStamp: 'mtf 7/14/2008 11:55' prior: 0!
I have no idea what this class is for. It is extensivily tested in SystemEditor-Tests, but it is actually only used two places in the code:
OrganizationEditor>>writeComment
MethodEditor>>logCompilationFrom:for:

I have not examined to see what it does -- Matthew Fulmer!

!SourceRepository class methodsFor: 'accessing' stamp: 'cwp 12/18/2005 14:59'!
createFakeFileStream
	| stream |
	stream := ReadWriteStream on: (String new: 100).
	stream nextPutAll: 'xxx'.
	stream position: 1.
	^ stream! !

!SourceRepository class methodsFor: 'instance creation' stamp: 'cwp 2/4/2006 12:46'!
default
	^ self forSourceFiles: SourceFiles! !

!SourceRepository class methodsFor: 'instance creation' stamp: 'cwp 12/17/2005 06:27'!
forSourceFiles: anArray
	^ self new setSourceFiles: anArray! !

!SourceRepository class methodsFor: 'instance creation' stamp: 'cwp 12/18/2005 15:00'!
newInternal
	^ self forSourceFiles: (Array 
							with: self createFakeFileStream
							with: self createFakeFileStream).! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 2/4/2006 12:32'!
assureStartupStampLogged
	SmalltalkImage current assureStartupStampLogged.! !

!SourceRepository methodsFor: 'accessing' stamp: 'cwp 12/17/2005 06:35'!
changesIndex
	^ 2! !

!SourceRepository methodsFor: 'accessing' stamp: 'cwp 12/17/2005 06:35'!
changesStream
	^ files at: self changesIndex! !

!SourceRepository methodsFor: 'comparing' stamp: 'cwp 2/4/2006 12:44'!
hash
	^ self sourcesStream identityHash bitXor: self changesStream identityHash! !

!SourceRepository methodsFor: 'initialize-release' stamp: 'cwp 12/17/2005 06:27'!
setSourceFiles: anArray
	files := anArray! !

!SourceRepository methodsFor: 'accessing' stamp: 'cwp 2/4/2006 12:42'!
sourcesIndex
	^ 1! !

!SourceRepository methodsFor: 'accessing' stamp: 'cwp 2/4/2006 12:43'!
sourcesStream
	^ files at: self sourcesIndex! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 1/29/2006 21:32'!
storeComment: aByteString forClass: aClass category: aByteString3 stamped: aByteString4 replacing: aRemoteString 
	self shouldBeImplemented! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 3/3/2006 01:39'!
storeComment: commentString forClass: aClass stamped: stampString prior: aSourcePointer
	| remote stream |
	self assureStartupStampLogged.
	stream := self changesStream.
	stream
		setToEnd;
		cr;
		nextPut: $!!;
		nextPutAll: aClass name;
		nextPutAll: ' commentStamp: ';
		print: stampString;
		nextPutAll: ' prior: ';
		print: aSourcePointer;
		nextPut: $!!;
		cr.
	remote := RemoteString 
				newString: commentString
				onFileNumber: self changesIndex 
				toFile: stream.
		^ remote! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 12/18/2005 14:05'!
storeComment: commentString forClass: aClass stamped: stampString replacing: aRemoteString
	^ self 
		storeComment: commentString
		forClass: aClass
		stamped: stampString
		prior: aRemoteString sourcePointer! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 12/17/2005 06:47'!
storeComment: commentString forClass: aClass stamped: stampString
	^ self storeComment: commentString forClass: aClass stamped: stampString prior: 0! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 3/3/2006 01:35'!
storeMethod: sourceString forClass: aClass category: catString stamped: stampString prior: sourcePointer
	| remote stream |
	self assureStartupStampLogged.
	stream := self changesStream.
	stream
		setToEnd;
		cr;
		nextPut: $!!;
		nextPutAll: aClass name;
		nextPutAll: ' methodsFor: ';
		print: catString;
		nextPutAll: ' stamp: ';
		print: stampString.
	sourcePointer ifNotNil: [stream nextPutAll: ' prior: '; print: sourcePointer].
	stream
		nextPut: $!!; cr.
	remote := RemoteString 
				newString: sourceString
				onFileNumber: self changesIndex 
				toFile: stream.
	stream space; nextPut: $!!.
		^ remote! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 1/29/2006 21:49'!
storeMethod: sourceString forClass: aClass category: catString 
stamped: stampString replacing: aRemoteString
	^ self
		storeMethod: sourceString 
		forClass: aClass 
		category: catString 
		stamped: stampString
		prior: aRemoteString sourcePointer.! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 3/3/2006 01:34'!
storeMethod: sourceString forClass: aClass category: catString stamped: stampString 
	^ self storeMethod: sourceString forClass: aClass category: catString stamped: stampString prior: nil! !

!SourceRepository methodsFor: 'comparing' stamp: 'cwp 2/4/2006 12:44'!
= other
	self class == other class ifFalse: [^ false].
	self sourcesStream == other sourcesStream ifFalse: [^ false].
	self changesStream == other changesStream ifFalse: [^ false].
	^ true! !

!CompiledMethod class methodsFor: '*systemeditor-migrating' stamp: 'cwp 3/19/2006 19:54'!
edInstanceBasedOn: oldInstance

	| size |
	size := oldInstance endPC - oldInstance initialPC + 1 + oldInstance trailer size.
	^ self newMethod: size header: oldInstance header.
! !

CategorizerTest subclass: #CategoryEditorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!CategoryEditorTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 20:24'!
setUp
	super setUp.
	categorizer := TestCategoryEditor on: categorizer! !

!CategoryEditorTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 14:51'!
tearDown
	self assert: categorizer subject printString =
'(''as yet unclassified'' d e)
(''abc'' a b c)
(''unreal'')
'! !

TestCase subclass: #ClassEditorTest
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

ClassEditorTest subclass: #ClassEditorBuildTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 4/14/2006 18:47'!
test01NoChange
	"Since there was no change, we don't have to rebuild the class at all."

	| migration |
	migration := MigrationTransaction new.
	editor edPrepareMigration: migration.
	self assert: migration isEmpty! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 4/14/2006 18:50'!
test02AddInstVar
	
	| result |
	editor addInstVarName: 'gamma'.
	result := editor product.
	self deny: result == editor subject.
	self assert: result name = editor subject name.
	self assert: result instVarNames = editor instVarNames.
	self assert: result format = editor format.
! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:10'!
test03Superclass
	
	| result |
	editor superclass: self class.
	result := editor product.
	self deny: result == editor subject.
	self assert: result superclass == self class.! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:10'!
test04AddClassVar
	
	| result |
	editor addClassVarName: 'Gimel'.
	result := editor product.
	self deny: result == editor subject.
	self assert: result classVarNames = editor classVarNames.
! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:10'!
test05RemoveClassVar
	
	| result |
	editor removeClassVarName: 'Beth'.
	result := editor product.
	self deny: result == editor subject.
	self assert: result classVarNames = editor classVarNames.! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:10'!
test06PreserveClassVar
	
	| result object |
	object := Object new.
	editor removeClassVarName: 'Beth'.
	editor subject aleph: object.
	result := editor product.
	self deny: result == editor subject.
	self assert: result aleph == object! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:11'!
test07AddSharedPool
	
	| result pool |
	pool := Dictionary new.
	editor addSharedPool: pool.
	result := editor product.
	self deny: result == editor subject.
	self assert: (result sharedPools includes: pool).
	self deny: (editor subject sharedPools includes: pool).! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:11'!
test08RemoveSharedPool
	
	| result |
	editor removeSharedPool: EditorSubjectPool.
	result := editor product.
	self deny: result == editor subject.
	self deny: (result sharedPools includes: EditorSubjectPool).
	self assert: (editor subject sharedPools includes: EditorSubjectPool).! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:11'!
test09ExistingMethods
	
	| result |
	editor compile: 'waw ^ 3' classified: 'phoenician'.
	result := editor product.
	self assert: (result compiledMethodAt: #het) == (editor subject compiledMethodAt: #het)! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/26/2005 22:25'!
test10MethodRemoved
	
	| dict |
	editor removeSelector: #het.
	dict := editor methodDictionary.
	self assert: dict isEmpty! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 4/15/2006 18:35'!
test11MethodAdded
	
	| dict selector |
	selector := editor 
				compile: 'zayin ^ 2'
				classified: 'numbers'
				withStamp: 'cwp 5/30/2004 09:27'
				notifying: nil
				logSource: true.
	dict := editor methodDictionary.
	self deny: editor edRequiresBuild.
	self assert: (dict at: #zayin) isCompiledMethod.
	self assert: selector == #zayin
! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/30/2005 17:07'!
test12MethodRecompilation
	
	| dict |
	editor addInstVarName: 'gamma'.
	dict := editor methodDictionary.
	self deny: (dict at: #het) == (editor subject compiledMethodAt: #het).
! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:11'!
test13NoSubject
	
	| result |
	editor := ClassEditor forNewClassNamed: #EditorSubject2.
	result := editor product.
	self assert: result name = #EditorSubject2! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 3/16/2006 19:11'!
test14SuperclassEditedInstVars
	
	| result subeditor |
	editor addInstVarName: 'gamma'.
	subeditor := ClassEditor forNewClassNamed: #EditorSubjectSub.
	subeditor superclass: editor.
	subeditor addInstVarName: 'delta'.
	result := subeditor product.
	self assert: result allInstVarNames = (editor subject instVarNames, #('gamma' 'delta')).

! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 4/15/2006 00:26'!
test15CommentChanged
	
	| migration |
	editor classComment: 'A new comment' stamp: 'cwp 12/18/2005 15:47'.

	migration := MigrationTransaction new.
	editor edPrepareMigration: migration.
	self assert: migration migrators size = 1.
	self assert: (migration migrators first origin isKindOf: ClassOrganizer)! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 5/28/2006 01:31'!
test16CommentNotChanged
	
	| migration |
	editor comment.

	migration := MigrationTransaction new.
	editor edPrepareMigration: migration.
	self assert: migration migrators isEmpty.! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 5/28/2006 01:44'!
test17NewMethodsAreClassified
	
	| migration organizer |
	editor 
		compile: 'zayin ^ 2'
		classified: 'numbers'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil
		logSource: true.
	
	migration := MigrationTransaction new.
	editor edPrepareMigration: migration.
	organizer := migration destinationFor: editor subject organization.
	self assert: (organizer categoryOfElement: #zayin) = 'numbers'! !

ClassEditorTest subclass: #ClassEditorOntologyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:06'!
test01Creation
	"This invariant allows editors and their subject classes to use
	exactly the same reflection protocol, so that the only modification
	required to allow tools to modify the system atomically is to send
	#commit when all modifications are complete."

	self assert: editor subject == EditorSubject.
	self assert: editor class subject == EditorSubject class! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:06'!
test02Printing
	"This makes debugging easier."
	
	self assert: editor printString = 'a ClassEditor on: EditorSubject'.
	self assert: editor class printString = 'a MetaclassEditor on: EditorSubject'.! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 07:47'!
test03ClassReferences

	"The instance is a strange beast. Its class is created on the fly, and is
	both an instance of MetaclassEditor and a subclass of ClassEditor. 
	See #testCreation." 
	self assert: editor class class == MetaclassEditor.
	self assert: (editor class instVarNamed: 'superclass') == ClassEditor.


	"editor is an instance of a light-weight class, so we don't register it
	with its superclass. In fact, the only thing that keeps it from being
	garbage collected is the class pointer from editor."
	self deny: (ClassEditor subclasses includes: editor class).

! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:07'!
test04IsKindOfClass
	"A class and its editor are polymorphic, but this kind of
	explicit test can tell the difference."

	self deny: (editor class isKindOf: Class)! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:24'!
test05ClassPool
	"To have DoIts be executed in the right context by a debugger or inspector,
	the generated class needs to be adoptable by FakeClassPool."

	self 
		shouldnt: [FakeClassPool adopt: editor class]
		raise: Error.! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 11/3/2005 23:30'!
test06Format
	"Despite having an instance variable called 'format' 
	MetaclassEditors should answer the format of the 
	subject's metaclass, not the format of ClassEditor class."

	self assert: editor class format = EditorSubject class format! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 15:57'!
test07MetaSuperclass
	"MetaclassEditor's ivar called 'superclass' should point to ClassEditor,
	but the method #superclass return an editor on the superclass of the
	editor's subject."

	self assert: editor class superclass subject == EditorSubject class superclass! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 15:50'!
test08Superclass
	"#superclass should return an editor on the subject's superclass"

	self assert: editor superclass subject == EditorSubject superclass! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 12/11/2005 13:27'!
test09Organization

	self assert: editor organization subject == editor subject organization! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:03'!
test10CompiledMethodAt
	"Compiled methods are immutable, so we don't wrap them for editing."

	self assert: (editor compiledMethodAt: #het) == (editor subject compiledMethodAt: #het)! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 1/14/2006 18:03'!
test11MetaOrganization

	self assert: editor class organization subject == editor class subject organization! !

ClassEditorTest subclass: #ClassEditorProtocolTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:40'!
test01InstVarNames
	self assert: editor instVarNames = editor subject instVarNames! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:36'!
test02AddInstVarName
	editor addInstVarName: 'gamma'.
	self assert: editor instVarNames = (editor subject instVarNames copyWith: 'gamma')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:58'!
test03RemoveClassVarName
	editor removeClassVarName: 'Beth'.
	self assert: editor classVarNames = (editor subject classVarNames copyWithout: 'Beth')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:40'!
test04ClassVarNames
	self assert: editor classVarNames = editor subject classVarNames! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 08:23'!
test05AddClassVarName
	editor addClassVarName: #Gimel.
	self assert: editor classVarNames = (editor subject classVarNames copyWith: #Gimel)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:59'!
test06RemoveClassVarName
	editor removeClassVarName: 'Beth'.
	self assert: editor classVarNames = (editor subject classVarNames copyWithout: 'Beth')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 16:00'!
test07SharedPools
	self assert: editor sharedPools = editor subject sharedPools! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 16:47'!
test08AddSharedPool
	| pool |
	pool := Dictionary new.
	editor addSharedPool: pool.
	self assert: editor sharedPools = (editor subject sharedPools copyWith: pool)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 07:51'!
test09RemoveSharedPool
	editor removeSharedPool: EditorSubjectPool.
	self assert: editor sharedPools = (editor subject sharedPools copyWithout: EditorSubjectPool)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 16:47'!
test10Superclass
	editor superclass: self class.
	self shouldnt: [editor edClassFormat] raise: MessageNotUnderstood.
	self assert: editor superclass subject == self class.! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 08:27'!
test11ClassVarNamesInterned
	editor addClassVarName: 'Gimel'.
	self assert: (editor classVarNames anySatisfy: [:ea | ea == #Gimel])! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/26/2005 08:40'!
test12AddSelector
	self deny: (editor includesSelector: #zayin).
	editor 
		compile: 'zayin ^ 1'
		classified: 'numbers'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil
		logSource: true.
	self assert: (editor includesSelector: #zayin)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/28/2005 08:13'!
test13RemoveSelector
	editor removeSelector: #het.
	self deny: (editor includesSelector: #het)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/26/2005 08:40'!
test14AddAndRemoveSelector
	editor 
		compile: 'zayin ^ 1'
		classified: 'numbers'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil
		logSource: true.
	editor removeSelector: #zayin.
	self deny: (editor includesSelector: #zayin).
	! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/28/2005 08:16'!
test15RemoveNewSelector
	editor removeSelector: #zayin.
	self deny: (editor includesSelector: #zayin)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/28/2005 22:06'!
test16TypeOfClass
	editor typeOfClass = #normal! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 17:13'!
test17Format

	| format |
	format := ClassFormat fromBits: editor format.
	self assert: format instSize = editor subject instSize.
	self assert: format instSpec = editor subject instSpec.
	self assert: format isBits = editor subject isBits.
	self assert: format isBytes = editor subject isBytes.
	self assert: format isFixed = editor subject isFixed.
	self assert: format isPointers = editor subject isPointers.
	self assert: format isVariable = editor subject isVariable.
	self assert: format isWeak = editor subject isWeak.
	self assert: format isWords = editor subject isWords.! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/29/2005 10:29'!
test18NewFormat
	| superFormat editorFormat |
	editor typeOfClass: #variable.
	superFormat := ClassFormat fromBits: EditorSubject superclass format.
	editorFormat := superFormat indexedSubclass: editor instVarNames size.
	self assert: editor format = editorFormat bits! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/29/2005 10:27'!
test19BeBytes
	editor typeOfClass: #bytes.
	self assert: editor typeOfClass == #bytes.
	self assert: editor isBytes.
	self assert: editor isBits.
	self assert: editor isVariable.
	self deny: editor isPointers.! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/29/2005 10:28'!
test20BeWords
	editor typeOfClass: #words.
	self assert: editor typeOfClass == #words.
	self assert: editor isWords.
	self assert: editor isBits.
	self assert: editor isVariable.
	self deny: editor isPointers.! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/29/2005 10:33'!
test21BeWeak
	editor typeOfClass: #weak.
	self assert: editor typeOfClass == #weak.
	self assert: editor isWords.
	self assert: editor isVariable.
	self assert: editor isPointers.
	self deny: editor isBits.
! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/29/2005 10:34'!
test22BeVariable
	editor typeOfClass: #variable.
	self assert: editor typeOfClass == #variable.
	self assert: editor isWords.
	self assert: editor isVariable.
	self assert: editor isPointers.
	self deny: editor isWeak.
	self deny: editor isBits.
! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/29/2005 10:35'!
test23BeNormal
	editor typeOfClass: #normal.
	self assert: editor typeOfClass == #normal.
	self assert: editor isPointers.
	self deny: editor isWords.
	self deny: editor isVariable.
	self deny: editor isWeak.
	self deny: editor isBits.
! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/29/2005 23:01'!
test24CompileClassifiedWithStampNotifying
	self deny: (editor includesSelector: #zayin).
	editor 
		compile: 'zayin ^ 1'
		classified: 'numbers'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil.
	self assert: (editor includesSelector: #zayin)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/29/2005 23:04'!
test25CompileClassifiedNotifying
	self deny: (editor includesSelector: #zayin).
	editor 
		compile: 'zayin ^ 1'
		classified: 'numbers'
		notifying: nil.
	self assert: (editor includesSelector: #zayin)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/30/2005 10:01'!
test26CompileClassified
	self deny: (editor includesSelector: #zayin).
	editor 
		compile: 'zayin ^ 1'
		classified: 'numbers'.
	self assert: (editor includesSelector: #zayin)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/3/2005 07:46'!
test27Subclass
	| sub |
	sub := editor
			subclass: #EditorSubjectSubclass
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: 'SystemEditor-Tests'.
	self assert: sub subject isNil.
	self assert: sub name = #EditorSubjectSubclass.
	self assert: sub instVarNames isEmpty.
	self assert: sub classVarNames isEmpty.
	self assert: sub sharedPools isEmpty.! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/12/2005 21:00'!
test28InstSize
	| sub |
	sub := editor
			subclass: #EditorSubjectSubclass
			instanceVariableNames: 'gamma'
			classVariableNames: ''
			poolDictionaries: ''
			category: 'SystemEditor-Tests'.
	self assert: sub instSize = 3! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/12/2005 21:35'!
test29InstVarsArentDuplicated
	editor addInstVarName: 'gamma'.
	editor addInstVarName: 'gamma'.
	self assert: editor instVarNames = (editor subject instVarNames copyWith: 'gamma')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/12/2005 21:39'!
test30ClassVarsArentDuplicated
	editor addClassVarName: 'Gimel'.
	editor addClassVarName: 'Gimel'.
	self assert: editor classVarNames = (editor subject classVarNames copyWith: 'Gimel')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 17:08'!
test31AllInstVarNames
	
	| subeditor |
	editor addInstVarName: 'gamma'.
	subeditor := ClassEditor forNewClassNamed: #EditorSubjectSub.
	subeditor superclass: editor.
	subeditor addInstVarName: 'delta'.
	self assert: subeditor allInstVarNames = (editor subject instVarNames, #('gamma' 'delta')).

! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 17:18'!
test32FormatWithEditedSuperclass

	| format supereditor |
	supereditor := (editor system at: #Object)
					subclass: #EditorSubjectSub 
					instanceVariableNames: 'gamma'
					classVariableNames: ''
					poolDictionaries: ''
					category: 'Testing'.
	editor superclass: supereditor.
	format := ClassFormat fromBits: editor format.
	self assert: format instSize = (supereditor instSize + editor instVarNames size)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 17:20'!
test33InstSizeWithEditedSuperclass

	| supereditor |
	supereditor := (editor system at: #Object)
					subclass: #EditorSubjectSub 
					instanceVariableNames: 'gamma'
					classVariableNames: ''
					poolDictionaries: ''
					category: 'Testing'.
	editor superclass: supereditor.
	self assert: editor instSize = (supereditor instSize + editor instVarNames size)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/23/2005 22:06'!
test34Subclasses

	| supereditor |
	supereditor := (editor system at: #Object)
					subclass: #EditorSubjectSuper 
					instanceVariableNames: ''
					classVariableNames: ''
					poolDictionaries: ''
					category: 'Testing'.
	editor superclass: supereditor.
	self assert: supereditor subclasses = (Array with: editor)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/1/2005 22:24'!
test35IsBehavior

	self assert: editor isBehavior! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/2/2005 22:49'!
test36InstanceVariablesString

	self assert: editor instanceVariablesString = editor subject instanceVariablesString! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/2/2005 22:55'!
test37ClassVariablesString

	self assert: editor classVariablesString = editor subject classVariablesString! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/2/2005 22:59'!
test38SharedPoolsString

	self assert: editor sharedPoolsString = editor subject sharedPoolsString! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/3/2005 09:39'!
test39Category

	self assert: editor category = editor subject category! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/3/2005 09:40'!
test40SetCategory

	editor category: #'Test-Category'.
	self assert: editor category = #'Test-Category'! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/11/2005 22:02'!
test41HasComment

	self deny: editor hasComment! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/11/2005 22:04'!
test42Comment
	self assert: (editor comment beginsWith: 'Main comment stating the purpose')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/11/2005 22:34'!
test43SetClassComment
	editor classComment: 'class comment' stamp: 'cwp 12/11/2005 22:00'.
	editor comment = 'class comment'! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:13'!
test44SourceCodeAt
	self assert: (editor sourceCodeAt: #het) = (editor subject sourceCodeAt: #het)! !

!ClassEditorTest class methodsFor: 'testing' stamp: 'cwp 11/10/2005 08:14'!
isAbstract
	^ self name == #ClassEditorTest! !

!ClassEditorTest methodsFor: 'running' stamp: 'cwp 10/23/2005 15:29'!
setUp
	editor := ClassEditor on: EditorSubject! !

ClassEditorTest subclass: #ClassEditorValidationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/12/2005 21:58'!
test01NoChangeIsValid
	self assert: editor edIsValid ! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:14'!
test02BitsWithVariablesIsInvalid
	editor typeOfClass: #bytes.
	self should: [editor validate] raise: InvalidClassFormat! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/12/2005 21:58'!
test03SameTypeIsValid
	editor addInstVarName: 'gamma'.
	self assert: editor edIsValid ! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:15'!
test04TooManyInstVars
	1 to: 255 do: [:i | editor addInstVarName: i asString].
	self should: [editor validate] raise: InvalidClassFormat! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:11'!
test05MultiplyDefinedInstVars
	"See ClassEditorProtocolTest>>test29InstVarsArentDuplicated"

	editor
		addInstVarName: 'gamma';
		addInstVarName: 'gamma'.
	self assert: editor edIsValid! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:16'!
test06InstVarNamedSelf

	editor addInstVarName: 'self'.
	self should: [editor validate] raise: IllegalVariableName.! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:27'!
test07InstVarNamedSuper

	editor addInstVarName: 'super'.
	self should: [editor validate] raise: IllegalVariableName.! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:27'!
test08InstVarNamedThisContext

	editor addInstVarName: 'thisContext'.
	self should: [editor validate] raise: IllegalVariableName.! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:28'!
test09InstVarNamedTrue

	editor addInstVarName: 'true'.
	self should: [editor validate] raise: IllegalVariableName.! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:29'!
test10InstVarNamedFalse

	editor addInstVarName: 'false'.
	self should: [editor validate] raise: IllegalVariableName.! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:29'!
test11InstVarNamedNil

	editor addInstVarName: 'nil'.
	self should: [editor validate] raise: IllegalVariableName.! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 18:39'!
test12InstVarDefinedInSuperclass

	| subeditor |
	subeditor := editor
				subclass: #EditorSubjectSub
				instanceVariableNames: 'alpha'
				classVariableNames: ''
				poolDictionaries: ''
				category: 'Testing'.
	self should: [subeditor validate] raise: IllegalVariableName.! !

!ClassEditorValidationTest methodsFor: 'tests' stamp: 'cwp 11/23/2005 22:20'!
test13InstVarDefinedInSubclass

	editor
		subclass: #EditorSubjectSub
		instanceVariableNames: 'gamma'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Testing'.
	editor addInstVarName: 'gamma'.
	self should: [editor validate] raise: IllegalVariableName.! !

TestCase subclass: #ClassFormatTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 11/23/2005 22:34'!
testBitsWithInstSize
	| format |
	format := ClassFormat bytes.
	self should: [format withInstSize: 2] raise: InvalidClassFormat! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testBytes
	| format |
	format := ClassFormat bytes.
	self assert: format instSize = 0.
	self assert: format isVariable.
	self assert: format isBits.
	self assert: format isBytes.
	self deny: format isFixed.
	self deny: format isPointers.
	self deny: format isWords.
	self deny: format isWeak
! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testIndexedSubclass
	| superFormat subFormat |
	superFormat := ClassFormat named: 3.
	subFormat := superFormat indexedSubclass: 2.
	self assert: superFormat instSize = 3.
	self assert: subFormat instSize = 5.
	self assert: subFormat isVariable.
	self assert: subFormat isPointers.
	self assert: subFormat isWords.
	self deny: subFormat isFixed.
	self deny: subFormat isBits.
	self deny: subFormat isBytes.
	self deny: subFormat isWeak! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testIndexedWithInstSize
	| old new |
	old := ClassFormat indexed: 3.
	new := old withInstSize: 5.
	self assert: new instSize = 5.
	self assert: new instSpec = old instSpec.
	self assert: new cClassIndex = old cClassIndex! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:27'!
testIndexed
	| format |
	format := ClassFormat indexed.
	self assert: format instSize = 0.
	self assert: format isPointers.
	self assert: format isWords.
	self assert: format isVariable.
	self deny: format isFixed.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak
! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testNamedAndIndexed
	| format |
	format := ClassFormat indexed: 3.
	self assert: format instSize = 3.
	self assert: format isPointers.
	self assert: format isWords.
	self assert: format isVariable.
	self deny: format isFixed.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak
! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testNamedSubclass
	| superFormat subFormat |
	superFormat := ClassFormat named: 3.
	subFormat := superFormat namedSubclass: 2.
	self assert: superFormat instSize = 3.
	self assert: subFormat instSize = 5.
	self assert: subFormat isFixed.
	self assert: subFormat isPointers.
	self assert: subFormat isWords.
	self deny: subFormat isVariable.
	self deny: subFormat isBits.
	self deny: subFormat isBytes.
	self deny: subFormat isWeak! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testNamedWithInstSize
	| old new |
	old := ClassFormat named: 3.
	new := old withInstSize: 5.
	self assert: new instSize = 5.
	self assert: new instSpec = old instSpec.
	self assert: new cClassIndex = old cClassIndex! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:27'!
testNamed
	| format |
	format := ClassFormat named: 3.
	self assert: format instSize = 3.
	self assert: format isFixed.
	self assert: format isPointers.
	self assert: format isWords.
	self deny: format isVariable.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 12/11/2005 10:38'!
testNoStateSubclass
	| format superformat |
	superformat := ClassFormat noState.
	format := superformat namedSubclass: 0.
	self assert: format instSize = 0.
	self assert: format isFixed.
	self assert: format isPointers.
	self assert: format isWords.
	self deny: format isVariable.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak.
	self assert: superformat bits == format bits! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testNoState
	| format |
	format := ClassFormat noState.
	self assert: format instSize = 0.
	self assert: format isFixed.
	self assert: format isPointers.
	self assert: format isWords.
	self deny: format isVariable.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testWeak
	| format |
	format := ClassFormat weak.
	self assert: format instSize = 0.
	self assert: format isVariable.
	self assert: format isPointers.
	self assert: format isWords.
	self assert: format isWeak.
	self deny: format isFixed.
	self deny: format isBits.
	self deny: format isBytes.
! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testWords
	| format |
	format := ClassFormat words.
	self assert: format instSize = 0.
	self assert: format isVariable.
	self assert: format isBits.
	self assert: format isWords.
	self deny: format isFixed.
	self deny: format isPointers.
	self deny: format isBytes.
	self deny: format isWeak
! !

TestCase subclass: #EditorSubjectSubTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

EditorSubjectSubTest subclass: #ClassExporterTest
	instanceVariableNames: 'exporter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassExporterTest methodsFor: 'running' stamp: 'cwp 11/10/2005 08:08'!
setUp
	super setUp.
	exporter := ClassExporter on: Smalltalk.
! !

!ClassExporterTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 17:21'!
test01ClassIsAddedToEnvironment
	| class |
	class := self createClass.
	exporter addClass: class.
	exporter commit.
	self assert: (Smalltalk at: class name) == class
	! !

!ClassExporterTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 17:30'!
test02ClassIsAddedToSuperclass
	| class |
	class := self createClass.
	exporter addClass: class.
	exporter commit.
	self assert: (class superclass subclasses includes: class)
	! !

!ClassExporterTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 17:40'!
test03ExistingReferencesAreMaintained
	| class reference |
	reference := Smalltalk at: #EditorSubjectSub.
	class := self createClassNamed: #EditorSubjectSub.
	exporter addClass: class.
	exporter commit.
	self assert: (Smalltalk at: #EditorSubjectSub) == reference.! !

!ClassExporterTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 18:43'!
test04ClassKnowsItsEnvironment
	| class |
	class := self createClass.
	exporter addClass: class.
	exporter commit.
	self assert: class environment == exporter environment
	! !

!ClassExporterTest methodsFor: 'tests' stamp: 'cwp 12/1/2005 23:34'!
test05NewClassReplacesOld
	| class |
	class := self createClassNamed: #EditorSubjectSub.
	exporter addClass: class.
	exporter commit.
	self assert: ((Smalltalk at: #EditorSubjectSub) instVarNames includes: 'gamma')! !

!EditorSubjectSubTest class methodsFor: 'as yet unclassified' stamp: 'cwp 11/10/2005 08:12'!
isAbstract
	^ self name == #EditorSubjectSubTest! !

!EditorSubjectSubTest methodsFor: 'support' stamp: 'cwp 3/16/2006 19:13'!
createClassNamed: aSymbol 
	| classEditor |
	classEditor := (ClassEditor on: EditorSubject) 
				subclass: aSymbol
				instanceVariableNames: 'gamma'
				classVariableNames: ''
				poolDictionaries: ''
				category: 'SystemEditor-Tests'.
	^classEditor product! !

!EditorSubjectSubTest methodsFor: 'support' stamp: 'cwp 11/10/2005 08:09'!
createClass
	^ self createClassNamed: #EditorSubjectSub2! !

!EditorSubjectSubTest methodsFor: 'running' stamp: 'cwp 11/10/2005 08:21'!
setUp
	(Smalltalk at: #EditorSubject)
		subclass: #EditorSubjectSub
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'SystemEditor-Tests'! !

!EditorSubjectSubTest methodsFor: 'running' stamp: 'cwp 11/10/2005 08:21'!
tearDown
	(Smalltalk at: #EditorSubject)
		allSubclasses do: [:ea | ea removeFromSystem]! !

EditorSubjectSubTest subclass: #SystemEditorCommitTest
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!SystemEditorCommitTest methodsFor: 'running' stamp: 'cwp 11/10/2005 08:11'!
setUp
	super setUp.
	editor := SystemEditor new.
! !

!SystemEditorCommitTest methodsFor: 'tests' stamp: 'cwp 11/10/2005 08:15'!
test01MigratesInstances
	| inst classEditor class |
	class := editor subject at: #EditorSubjectSub.
	inst := class new.
	classEditor := editor at: #EditorSubjectSub.
	classEditor addInstVarName: 'gamma'.
	editor commit.
	self assert: (inst class instVarNames includes: 'gamma')! !

!SystemEditorCommitTest methodsFor: 'tests' stamp: 'cwp 12/3/2005 17:50'!
test02CategorizesClasses
	| classEditor |
	classEditor := editor at: #EditorSubjectSub.
	classEditor category: #'SystemEditor-Info'.
	editor commit.
	self assert: (SystemOrganization categoryOfElement: #EditorSubjectSub) == #'SystemEditor-Info'! !

!SystemEditorCommitTest methodsFor: 'tests' stamp: 'cwp 12/3/2005 21:19'!
test03RemovesClasses
	| classEditor |
	classEditor := editor at: #EditorSubjectSub.
	classEditor removeFromSystem.
	editor commit.
	self deny: (Smalltalk hasClassNamed: #EditorSubjectSub)! !

TestCase subclass: #InstanceMigratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!InstanceMigratorTest methodsFor: 'support' stamp: 'cwp 3/16/2006 19:47'!
createClassPair
	| editor oldClass newClass |
	editor := ClassEditor on: EditorSubject.
	editor compile: 'isInstanceOfNewClass ^ false' classified: 'testing'.
	oldClass := editor edBuild.
	editor compile: 'isInstanceOfNewClass ^ true' classified: 'testing'.
	newClass := editor edBuild.
	^ oldClass -> newClass! !

!InstanceMigratorTest methodsFor: 'tests' stamp: 'cwp 3/19/2006 19:46'!
testMigratingCompiledMethod
	| migrator oldInstance newInstance |
	migrator := InstanceMigrator from: CompiledMethod to: CompiledMethod.
	oldInstance := self class compiledMethodAt: #testMigratingCompiledMethod.
	newInstance := migrator migrate: oldInstance.
	self assert: newInstance class == CompiledMethod.
	self assert: newInstance basicSize = oldInstance basicSize.
	self assert: newInstance header = oldInstance header.! !

!InstanceMigratorTest methodsFor: 'tests' stamp: 'cwp 11/2/2005 20:06'!
testOneClassTransaction
	| mutator classes inst |
	classes := self createClassPair.
	inst := classes key new.
	mutator := MigrationTransaction new.
	mutator addMigrator: (InstanceMigrator from: classes key to: classes value).
	mutator commit.
	self assert: inst isInstanceOfNewClass! !

!InstanceMigratorTest methodsFor: 'tests' stamp: 'cwp 11/2/2005 20:06'!
testTwoClassTransaction
	| mutator classes inst1 inst2 |
	mutator := MigrationTransaction new.

	classes := self createClassPair.
	inst1 := classes key new.
	mutator addMigrator: (InstanceMigrator from: classes key to: classes value).

	classes := self createClassPair.
	inst2 := classes key new.
	mutator addMigrator: (InstanceMigrator from: classes key to: classes value).

	mutator commit.
	self assert: inst1 isInstanceOfNewClass.
	self assert: inst2 isInstanceOfNewClass! !

TestCase subclass: #MetaclassEditorTest
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

MetaclassEditorTest subclass: #MetaclassEditorBuildTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!MetaclassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 14:30'!
test01NoChange
	"We always rebuild the metaclass, since the class needs to be an instance of it."

	self deny: editor edBuild == editor subject.
	self deny: editor edRequiresRecompile! !

!MetaclassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 11/4/2005 22:05'!
test02AddInstVar
	
	| result |
	editor addInstVarName: 'gamma'.
	result := editor edBuild.
	self deny: result == editor subject.
	self assert: result instVarNames = editor instVarNames.
	self assert: result format = editor format.
! !

!MetaclassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 11/20/2005 16:53'!
test03Superclass
	
	| result |
	editor superclass: self class class.
	result := editor edBuild.
	self deny: result == editor subject.
	self assert: result superclass == self class class.! !

!MetaclassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 11:46'!
test04ExistingMethods
	
	| result |
	editor compile: 'pe ^ 2'.
	result := editor edBuild.
	self assert: (result compiledMethodAt: #ayin) == (editor subject compiledMethodAt: #ayin)! !

!MetaclassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 11:39'!
test05MethodRemoved
	
	| dict |
	editor removeSelector: #ayin.
	dict := editor methodDictionary.
	self deny: (dict includesKey: #ayin)! !

!MetaclassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 11:41'!
test06MethodAdded
	
	| dict |
	editor 
		compile: 'pe ^ -2'
		classified: 'letters'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil
		logSource: true.
	dict := editor methodDictionary.
	self assert: (dict at: #pe) isCompiledMethod.
! !

!MetaclassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 11:41'!
test07MethodRecompilation
	
	| dict |
	editor addInstVarName: 'psi'.
	dict := editor methodDictionary.
	self deny: (dict at: #ayin) == (editor subject compiledMethodAt: #ayin).
! !

!MetaclassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 11/5/2005 14:50'!
test09NoSubject
	
	| result |
	editor := (ClassEditor forNewClassNamed: #EditorSubect2) class.
	result := editor edBuild.
	self assert: result isMeta! !

MetaclassEditorTest subclass: #MetaclassEditorProtocolTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 22:50'!
test01InstVarNames
	self assert: editor instVarNames = editor subject instVarNames! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 23:14'!
test02AddInstVarName
	editor addInstVarName: 'psi'.
	self assert: editor instVarNames = (editor subject instVarNames copyWith: 'psi')! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/27/2005 08:31'!
test03RemoveInstVarName
	editor removeInstVarName: 'omega'.
	self assert: editor instVarNames = (editor subject instVarNames copyWithout: 'omega')! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 11/4/2005 21:13'!
test04Superclass
	
	| result |
	editor superclass: self class.
	result := editor edBuild.
	self deny: result == editor subject.
	self assert: result superclass == self class.! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:57'!
test05AddSelector
	self deny: (editor includesSelector: #zayin).
	editor 
		compile: 'zayin ^ 1'
		classified: 'numbers'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil
		logSource: true.
	self assert: (editor includesSelector: #zayin)! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:57'!
test06RemoveNewSelector
	editor removeSelector: #pe.
	self deny: (editor includesSelector: #pe)! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:57'!
test07RemoveSelector
	editor removeSelector: #ayin.
	self deny: (editor includesSelector: #ayin)! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:57'!
test08AddAndRemoveSelector
	editor 
		compile: 'pe ^ 1'
		classified: 'phoenician'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil
		logSource: true.
	editor removeSelector: #pe.
	self deny: (editor includesSelector: #pe).
	! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:57'!
test09CompiledMethodAt
	"Compiled methods are immutable, so we don't wrap them for editing."

	self assert: (editor compiledMethodAt: #ayin) == (editor subject compiledMethodAt: #ayin)! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:57'!
test10SourceCodeAt
	self assert: (editor sourceCodeAt: #ayin) = (editor subject sourceCodeAt: #ayin)! !

!MetaclassEditorTest class methodsFor: 'as yet unclassified' stamp: 'cwp 11/10/2005 08:15'!
isAbstract
	^ self name == #MetaclassEditorTest! !

!MetaclassEditorTest methodsFor: 'running' stamp: 'cwp 10/24/2005 07:57'!
setUp
	editor := (ClassEditor on: EditorSubject) class! !

TestCase subclass: #MethodEditorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!MethodEditorTest methodsFor: 'emulating' stamp: 'cwp 2/1/2006 20:12'!
sourcePointer
	^ 42! !

!MethodEditorTest methodsFor: 'tests' stamp: 'cwp 3/3/2006 14:58'!
test01FirstCompilationIsLogged
	| editor actual expected |
	editor := MethodEditor
				source: 'one ^ 1'
				classified: 'numbers'
				stamp: 'cwp 12/11/2005 21:58'
				notifying: nil
				logging: true.
	editor repository: SourceRepository newInternal.
	editor compileFrom: nil for: self class.
	actual := editor repository changesStream contents.
	expected := 'xxx
!!MethodEditorTest methodsFor: ''numbers'' stamp: ''cwp 12/11/2005 21:58''!!
one ^ 1!! !!'.
	self assert: actual = expected! !

!MethodEditorTest methodsFor: 'tests' stamp: 'cwp 1/31/2006 23:13'!
test02LoggingCanBeSupressed
	| editor actual |
	editor := MethodEditor
				source: 'one ^ 1'
				classified: 'numbers'
				stamp: 'cwp 12/11/2005 21:58'
				notifying: nil
				logging: false.
	editor repository: SourceRepository newInternal.
	editor compileFrom: nil for: self class.
	actual := editor repository changesStream contents.
	self assert: actual = 'xxx'! !

!MethodEditorTest methodsFor: 'tests' stamp: 'cwp 3/3/2006 15:00'!
test03RecompilationIsLogged
	| editor actual expected |
	editor := MethodEditor
				source: 'one ^ 1'
				classified: 'numbers'
				stamp: 'cwp 12/11/2005 21:58'
				notifying: nil
				logging: true.
	editor repository: SourceRepository newInternal.
	editor compileFrom: self for: self class.
	actual := editor repository changesStream contents.
	expected := 'xxx
!!MethodEditorTest methodsFor: ''numbers'' stamp: ''cwp 12/11/2005 21:58'' prior: 42!!
one ^ 1!! !!'.
	self assert: actual = expected! !

!MethodEditorTest methodsFor: 'tests' stamp: 'cwp 3/3/2006 15:00'!
test04SourcePointerIsSet
	| editor cm |
	editor := MethodEditor
			source: 'one ^ 1'
			classified: 'numbers'
			stamp: 'cwp 12/11/2005 21:58'
			notifying: nil
			logging: true.
	editor repository: SourceRepository newInternal.
	cm := editor compileFrom: self for: self class.
	self assert: cm sourcePointer = 33554517! !

!MethodEditorTest methodsFor: 'tests' stamp: 'cwp 2/1/2006 20:19'!
test05HasDefaultRepository
	| editor |
	editor := MethodEditor
			source: 'one ^ 1'
			classified: 'numbers'
			stamp: 'cwp 12/11/2005 21:58'
			notifying: nil
			logging: true.
	self assert: editor repository = SourceRepository default! !

TestCase subclass: #OrganizationEditorTest
	instanceVariableNames: 'editor stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!OrganizationEditorTest commentStamp: 'cwp 12/18/2005 15:47' prior: 0!
This comment must be present for the tests to pass.!

!OrganizationEditorTest methodsFor: 'asserting' stamp: 'cwp 12/19/2005 21:55'!
assertRemoteString: remote inRepository: repository matches: aString
	self assert: remote sourceFileNumber = 2.
	repository changesStream position: remote position.
	self assert: (repository changesStream next: aString size) = aString.
	repository changesStream next = $!!! !

!OrganizationEditorTest methodsFor: 'remote string' stamp: 'cwp 12/13/2005 21:58'!
basicFileStream
	^ stream := WriteStream on: (String new: 20)! !

!OrganizationEditorTest methodsFor: 'remote string' stamp: 'cwp 12/13/2005 21:55'!
position
	^ 42! !

!OrganizationEditorTest methodsFor: 'construction' stamp: 'cwp 12/18/2005 14:51'!
repository
	self shouldBeImplemented! !

!OrganizationEditorTest methodsFor: 'running' stamp: 'cwp 12/15/2005 22:18'!
setUp
	editor := OrganizationEditor for: EditorSubject! !

!OrganizationEditorTest methodsFor: 'remote string' stamp: 'cwp 12/13/2005 22:54'!
sourceFileNumber
	^ 2! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 21:26'!
test01SubjectCanBeSpecified
	editor := OrganizationEditor for: (ClassEditor on: EditorSubject).
	self assert: editor subject == EditorSubject organization! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 21:26'!
test02ClassComment
	editor := OrganizationEditor for: (ClassEditor on: self class).
	self assert: editor classComment asString = 'This comment must be present for the tests to pass.'! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 21:27'!
test03CommentStamp
	editor := OrganizationEditor for: (ClassEditor on: self class).
	self assert: editor commentStamp = OrganizationEditorTest organization commentStamp! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/15/2005 22:17'!
test04SetComment
	editor := OrganizationEditor for: EditorSubject.
	editor classComment: 'comment string' stamp: 'cwp 12/11/2005 21:58'.
	self assert: editor classComment = 'comment string'! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/15/2005 22:18'!
test05SetCommentStamp
	editor := OrganizationEditor for: EditorSubject.
	editor classComment: 'comment string' stamp: 'cwp 12/11/2005 21:58'.
	self assert: editor commentStamp = 'cwp 12/11/2005 21:58'.! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 21:27'!
test06BuildCopiesCategories
	| actual expected |
	editor := OrganizationEditor for: (ClassEditor on: self class).
	actual := editor build.
	expected := self class organization.
	self deny: actual == expected.
	self assert: actual printString = expected printString! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 21:55'!
test07BuildWritesCommentToChangesFile
	| remote repository result |
	repository := SourceRepository newInternal.
	editor := OrganizationEditor for: (ClassEditor on: self class)
				repository: repository.
	editor classComment: 'comment string' stamp: 'cwp 12/11/2005 21:58'.
	result := editor build.
	remote := result commentRemoteStr.
	self assertRemoteString: remote inRepository: repository matches: 'comment string'! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 22:23'!
test08NilCommentDoesntWriteToChangesFile
	| repository result |
	repository := SourceRepository newInternal.
	editor := OrganizationEditor for: (ClassEditor on: self class)
				repository: repository.
	editor classComment: nil stamp: nil.
	result := editor build.
	self assert: result commentRemoteStr isNil.
	self assert: repository changesStream size = 3! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 22:39'!
test09EmptyCommentWrittenToChangesFile
	| remote repository result |
	repository := SourceRepository newInternal.
	editor := OrganizationEditor for: (ClassEditor on: self class)
				repository: repository.
	editor classComment: '' stamp: 'cwp 12/11/2005 21:58'.
	result := editor build.
	remote := result commentRemoteStr.
	self assertRemoteString: remote inRepository: repository matches: ''! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 12/19/2005 23:10'!
test10CategoryOfElement
	editor := OrganizationEditor for: (ClassEditor on: self class).
	self assert: (editor categoryOfElement: #test10CategoryOfElement) = #tests.! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 2/1/2006 22:45'!
test11RemovingCommentCausesBuild
	| repository |
	repository := SourceRepository newInternal.
	editor := OrganizationEditor for: (ClassEditor on: self class)
				repository: repository.
	editor classComment: nil stamp: nil.
	self assert: editor edRequiresBuild! !

!OrganizationEditorTest methodsFor: 'tests' stamp: 'cwp 5/28/2006 01:11'!
test12ReadingCommentDoesntRequireBuild
	editor := OrganizationEditor for: (ClassEditor on: self class).
	editor classComment.
	self deny: editor edRequiresBuild! !

TestCase subclass: #SourceRepositoryTest
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!SourceRepositoryTest methodsFor: 'running' stamp: 'cwp 12/18/2005 15:01'!
setUp
	repository := SourceRepository newInternal! !

!SourceRepositoryTest methodsFor: 'tests' stamp: 'cwp 3/3/2006 01:38'!
test01StoreCommentClassStampedReplacing
	| remote |
	remote := repository 
		storeComment: 'This is a comment.'
		forClass: self class
		stamped: 'cwp 12/11/2005 21:58'
		replacing: (RemoteString newFileNumber: 1 position: 42).
	self assert: repository changesStream contents = 'xxx
!!SourceRepositoryTest commentStamp: ''cwp 12/11/2005 21:58'' prior: 16777258!!
This is a comment.!!'.

	self assert: remote sourceFileNumber = 2.
	self assert: remote position = 80! !

!SourceRepositoryTest methodsFor: 'tests' stamp: 'cwp 3/3/2006 01:38'!
test02StoreCommentForClassStamped
	| remote |
	remote := repository 
		storeComment: 'This is a comment.'
		forClass: self class
		stamped: 'cwp 12/11/2005 21:58'.

	self assert: repository changesStream contents = 'xxx
!!SourceRepositoryTest commentStamp: ''cwp 12/11/2005 21:58'' prior: 0!!
This is a comment.!!'.

	self assert: remote sourceFileNumber = 2.
	self assert: remote position = 73! !

!SourceRepositoryTest methodsFor: 'tests' stamp: 'cwp 3/3/2006 01:38'!
test03StoreCommentClassStampedPrior
	| remote |
	remote := repository 
		storeComment: 'This is a comment.'
		forClass: self class
		stamped: 'cwp 12/11/2005 21:58'
		prior: 42.
	self assert: repository changesStream contents = 'xxx
!!SourceRepositoryTest commentStamp: ''cwp 12/11/2005 21:58'' prior: 42!!
This is a comment.!!'.

	self assert: remote sourceFileNumber = 2.
	self assert: remote position = 74! !

!SourceRepositoryTest methodsFor: 'tests' stamp: 'cwp 3/3/2006 01:36'!
test04StoreMethodReplacing
	| remote actual expected |
	remote := repository 
				storeMethod: 'one ^ 1'
				forClass: self class
				category: 'numbers'
				stamped: 'cwp 12/11/2005 21:58'
				replacing: (RemoteString newFileNumber: 1 position: 42).
	actual := repository changesStream contents.
	expected := 'xxx
!!SourceRepositoryTest methodsFor: ''numbers'' stamp: ''cwp 12/11/2005 21:58'' prior: 16777258!!
one ^ 1!! !!'.
	self assert: actual = expected.
	self assert: remote sourceFileNumber = 2.
	self assert: remote position = 95! !

TestCase subclass: #SystemEditorTest
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!SystemEditorTest methodsFor: 'asserting' stamp: 'cwp 4/14/2006 16:37'!
assertClassesAreMigrated: classes 
	self assert: (self migratedClasses includesAllOf: classes)! !

!SystemEditorTest methodsFor: 'asserting' stamp: 'cwp 4/14/2006 16:37'!
denyClassesAreMigrated: classes 
	self deny: (self migratedClasses includesAnyOf: classes)! !

!SystemEditorTest methodsFor: 'asserting' stamp: 'cwp 4/14/2006 16:36'!
migratedClasses
	| migration |
	editor edExpandEditors.
	migration := MigrationTransaction new.
	editor edPrepareMigration: migration.
	^ migration migrators collect: [:ea | ea origin].! !

!SystemEditorTest methodsFor: 'running' stamp: 'cwp 10/28/2005 20:41'!
setUp
	editor := SystemEditor new.
! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 11/6/2005 00:21'!
test01SubjectCanBeSpecified
	editor := SystemEditor on: Smalltalk.
	self assert: editor subject == Smalltalk! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 11/6/2005 00:21'!
test02DefaultSubjectIsSmalltalk
	self assert: editor subject == Smalltalk! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 11/6/2005 00:21'!
test03AtCreatesClassEditor
	| classEditor |
	classEditor := editor at: #EditorSubject.
	self assert: classEditor subject = EditorSubject! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 11/6/2005 00:22'!
test04ClassEditorsAreCached
	| first second |
	first := editor at: #EditorSubject.
	second := editor at: #EditorSubject.
	self assert: first == second! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 11/6/2005 00:22'!
test05ClassEditorsKnowTheirSystemEditors
	| first |
	first := editor at: #EditorSubject.
	self assert: first system == editor! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/3/2005 20:58'!
test06AtDoesntCreateNewClass
	self should: [editor at: #EditorSubject2] raise: Error
	! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 4/14/2006 16:35'!
test07ReshapesSubclasses
	| superclass |
	superclass := editor at: #ClassEditorTest.
	superclass addInstVarName: 'gamma'.
	self assertClassesAreMigrated: ClassEditorTest subclasses! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 4/14/2006 16:38'!
test08DoesntRebuildSubclassesUnnecessarily
	| superclass |
	superclass := editor at: #ClassEditorTest.
	superclass compile: 'het ^ #one' classified: 'testing'.
	self denyClassesAreMigrated: ClassEditorTest subclasses.! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 4/14/2006 16:50'!
test09RecompilesSubclasses
	"This test isn't quite right. It should actually compare compiled methods."

	| superclass |
	superclass := editor at: #ClassEditorTest.
	superclass removeInstVarName: 'alpha'.
	self assertClassesAreMigrated: ClassEditorTest subclasses! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 11/12/2005 20:45'!
test10ValidatesBeforeCommitting
	| classEditor |
	classEditor := editor at: #EditorSubject.
	classEditor typeOfClass: #bytes.
	classEditor addInstVarName: 'test'.
	self 
		should: [editor commit]
		raise: InvalidClassFormat! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/1/2005 22:50'!
test11AtIfAbsentFindsExistingEditor
	| object |
	object := editor at: #Object.
	self assert: (editor at: #Object ifAbsent: [nil]) == object! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/1/2005 22:49'!
test12AtIfAbsentCreatesEditorForExistingClass
	self assert: (editor at: #Object ifAbsent: [nil]) subject == Object! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/1/2005 23:01'!
test13AtIfAbsentEvaluatesBlock
	| object |
	object := Object new.
	self assert: (editor at: #'Illegal Name' ifAbsent: [object]) == object! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/1/2005 23:05'!
test14AtIfAbsentDoesntWrapPools
	self assert: (editor at: #Undeclared ifAbsent: [0]) == Undeclared! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/2/2005 23:05'!
test15KeyAtIdentityValueFindPools
	self assert: (editor keyAtIdentityValue: Undeclared ifAbsent: [nil]) = #Undeclared! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/2/2005 23:09'!
test16KeyAtIdentityValueFindEditors
	| classEditor |
	classEditor := editor at: #Object.
	self assert: (editor keyAtIdentityValue: classEditor ifAbsent: [nil]) == #Object! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/3/2005 20:15'!
test17RemovedClassesAreAbsent
	| classEditor |
	classEditor := editor at: #EditorSubject.
	classEditor removeFromSystem.
	self assert: (editor at: #EditorSubject ifAbsent: [nil]) isNil! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 12/3/2005 21:05'!
test18RemovedClassesArentPresent
	| classEditor |
	classEditor := editor at: #EditorSubject.
	classEditor removeFromSystem.
	self should: [editor at: #EditorSubject] raise: Error.! !

!SystemEditorTest methodsFor: 'tests' stamp: 'cwp 3/3/2006 15:16'!
test19AtDoesntWrapPools
	self assert: (editor at: #Undeclared) == Undeclared! !

!UndefinedObject methodsFor: '*systemeditor-building' stamp: 'cwp 4/15/2006 20:16'!
edPrepareMigration: aMigrationTransaction for: anEditor! !

!UndefinedObject methodsFor: '*systemeditor-building' stamp: 'cwp 4/15/2006 20:09'!
edPrepareMigration: aMigrationTransaction! !

!RemoteString methodsFor: '*systemeditor-accessing' stamp: 'cwp 12/13/2005 21:52'!
basicFileStream
	(sourceFileNumber == nil or: [(SourceFiles at: sourceFileNumber) == nil]) ifTrue: [^ nil].
	^ SourceFiles at: sourceFileNumber.! !

Notification subclass: #InvalidSystemChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Exceptions'!

InvalidSystemChange subclass: #IllegalVariableName
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Exceptions'!

InvalidSystemChange subclass: #InvalidClassFormat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Exceptions'!

!Behavior methodsFor: '*systemeditor-migrating' stamp: 'cwp 3/19/2006 19:25'!
edInstanceBasedOn: oldInstance 
	| newInstance |
	self isVariable 
		ifTrue: [newInstance := self basicNew: oldInstance basicSize]
		ifFalse: [newInstance := self basicNew].
	^newInstance! !

Behavior subclass: #OldMetaclassEditor
	instanceVariableNames: 'editor superEditor instVarNames methods organization traitComposition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!OldMetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 7/28/2008 23:48'!
classClassEditor
"Answers the class of my instance-side editor"

	^ ClassEditor! !

!OldMetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 7/28/2008 23:49'!
new
	| instance |
	instance := self basicNew.
	instance
		superclass: self classClassEditor
		methodDictionary: MethodDictionary new
		format: self classClassEditor format.
	^ instance! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 11/16/2007 21:49'!
acceptsLoggingOfCompilation

	^ self instanceSide acceptsLoggingOfCompilation! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'cwp 11/4/2005 22:08'!
addInstVarName: aString 
	instVarNames ifNil: [instVarNames := self subject instVarNames].
	instVarNames := instVarNames copyWith: aString! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 7/22/2008 14:52'!
allInstVarNames
	 "specialized in order to enable debugger to show as self"
	
	self isDebuggingAsEditor ifTrue: [^ super allInstVarNames].
	^ self edSuperclass
		ifNil: [self instVarNames copy]
		ifNotNil: [self edSuperclass allInstVarNames, self instVarNames]! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 9/20/2007 12:34'!
browserDefinition: style
	^ self definition! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 18:21'!
classPool
	^ editor classPool! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 11/10/2007 00:29'!
classSide
	^self theMetaClass! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 11/10/2007 01:11'!
comment
	^ self instanceSide comment! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 12/19/2005 23:54'!
compiledMethodAt: aSelector
	^ self subjectDo: [:subject | subject compiledMethodAt: aSelector]! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'mtf 6/12/2008 12:20'!
compile: source classified: cat notifying: anObject
	^ self 
		compile: source
		classified: cat
		withStamp: Utilities changeStamp
		notifying: anObject
		logSource: self acceptsLoggingOfCompilation ! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'mtf 9/11/2007 23:22'!
compile: source classified: category withStamp: aString notifying: anObject logSource: aBoolean
	| cm |
	cm := MethodEditor
			source: source
			classified: category
			stamp: aString
			notifying: anObject
			logging: aBoolean.
	self methods add: cm.! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'mtf 6/12/2008 12:24'!
compile: source classified: cat withStamp: aString notifying: anObject
	^ self 
		compile: source
		classified: cat
		withStamp: aString
		notifying: anObject
		logSource: self acceptsLoggingOfCompilation ! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'mtf 11/16/2007 21:46'!
compile: source classified: cat
	^ self 
		compile: source
		classified: cat
		withStamp: Utilities changeStamp
		notifying: nil
		logSource: self acceptsLoggingOfCompilation ! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 2/1/2008 22:37'!
definitionST80
	^ String streamContents: 
		[:strm |
		strm print: self;
			crtab;
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString]! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 12/19/2007 00:54'!
definitionST80: isST80
	^ isST80 ifTrue: [self definitionST80] ifFalse: [self definition]! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 2/1/2008 22:39'!
definition
	^ self definitionST80! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 12:04'!
edBuild
	| result class |
	class := self subject ifNil: [Metaclass] ifNotNilDo: [:subject | subject class].
	result := class basicNew.
	"Create a temporary MethodDictionary to catch code written by SyntaxError dialogs. MethodDictionaryEditor will overwrite this. See MethodEditor>>compileFor:"
	result
		superclass: self edSuperclass
		methodDictionary: MethodDictionary new
		format: self format;		
		organization: self organization build.
	result setInstVarNames: self instVarNames.
	^ result! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/6/2008 01:44'!
edClassFormat
	^ ClassFormat
		size: self instSize
		type: self typeOfClass
		index: self indexIfCompact! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'mtf 9/11/2007 23:52'!
edInitializeForNewClass
	superEditor := self system edEditorFor: Object class.
	instVarNames := Array new.
	methods := MethodDictionaryEditor for: self! !

!OldMetaclassEditor methodsFor: 'testing' stamp: 'cwp 11/20/2005 16:29'!
edIsEditor
	^ true! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'mtf 9/17/2007 11:38'!
edMethodAt: aSelector ifAbsent: aBlock
	^ self methods at: aSelector ifAbsent: aBlock! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'mtf 9/12/2007 00:04'!
edMethodAt: aSelector
	^ self methods at: aSelector! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'cwp 1/14/2006 18:58'!
edMethodsDo: aBlock
	methods ifNotNil: [methods do: aBlock]! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'cwp 4/15/2006 21:00'!
edRequiresBuild
	^ (superEditor ~~ nil) | (instVarNames ~~ nil) | (methods ~~ nil)! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'cwp 11/5/2005 11:43'!
edRequiresRecompile
	^ instVarNames ~~ nil! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'cwp 11/7/2005 19:51'!
edRequiresSubclassRebuild
	^ self subject
		ifNotNilDo: [:subject | subject format ~= self format]
		ifNil: [false]! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'mtf 7/22/2008 12:16'!
edSuperclass
	^ superEditor
		ifNil: [self subject superclass]
		ifNotNil: [superEditor product]! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 11/10/2007 00:45'!
environment
	^ self system! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 7/29/2008 08:50'!
format
"Override the implemenation in Behavior. That will return the format of ClassEditor, but we want the format of the subject's meta class"

	^ self edClassFormat bits! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 11/10/2007 01:10'!
hasComment
	^ self instanceSide hasComment! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 10/28/2005 08:11'!
includesSelector: aSelector
	^ (methods ifNil: [self subject] ifNotNil: [methods])
		includesSelector: aSelector! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 7/22/2008 12:34'!
indexIfCompact
	self subject ifNotNil: [^ self subject indexIfCompact].
	^ 0! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 11/10/2007 00:29'!
instanceSide
	^ self theNonMetaClass! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 4/18/2008 22:23'!
instanceVariableNames: instString
	instVarNames := Scanner new scanFieldNames: instString! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 2/4/2008 22:16'!
instanceVariablesString
	^ String streamContents: 
		[:stream | 
		self allInstVarNames
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream nextPut: $ ]]! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 7/22/2008 12:27'!
instSize
	"Override the implemenation in Behavior, so that when inspecting the
	ClassEditor, it will look like a regular Class"

	self isDebuggingAsEditor ifTrue: [^ super instSize].
	^ self instVarNames size + (self edSuperclass
		ifNil: [0] ifNotNilDo: [:ed | ed instSize])! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 9/4/2007 16:59'!
instVarNames
	"Override the implemenation in Behavior. That will return the inst vars of 
	ClassEditor, but we want the inst vars of the subject's class."

	self isDebuggingAsEditor ifTrue: [^ super instVarNames].
	^ instVarNames ifNil: [self subject instVarNames]! !

!OldMetaclassEditor methodsFor: 'debugging' stamp: 'mtf 9/18/2008 10:47'!
isDebuggingAsEditor
	"To be transparent enough to fool most existing software,
	MetaclassEditors must answer their subject's instance variables when
	sent the message allInstVarNames. However, this message is also used by
	the debugger, inspector, and explorer, when inspecting ClassEditors, and
	an inspector should see the real instance variables of ClassEditor rather
	than those of the MetaclassEditor's subject. So there is a conflict of
	interest already.
	
	But it gets worse. If you do open an inspector on an object that is
	reporting wrong information about it's instance variables, Bad things
	happen. Specifically, the inspector calls instVarAt: on an instance
	variable that does not exist, leading to an error every time the display
	is updated. Less severe, instVarAt: will answer nonsensical. 
	
	So I took the instance variable proxying behavior all the way so that it
	successfully fools the debugger, inspector, and explorer into seeing
	proxied instance variables (see the senders af this message if you want
	to know how). But there is still the original problem that you, the
	programmer want to see the real state of ClassEditor. That is where this
	method comes in.
	
	If this method returns false, everybody sees the proxied instance
	variables. This is the correct behavior. If this method returns false,
	everyone sees the true instance variables of ClassEditor. This is
	incorrect behavior, and breaks the Unit tests, but it is invaluable if
	you are debugging ClassEditor"

	^ self environment debug! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 9/13/2007 19:54'!
isMeta
	^ true! !

!OldMetaclassEditor methodsFor: 'testing' stamp: 'mtf 10/6/2008 13:47'!
isMethodsChanged
	methods ifNil: [^ false].
	^ methods isChanged! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 10/9/2007 14:50'!
isUniClass
	"The debugger sometimes sends this message"
	^ false! !

!OldMetaclassEditor methodsFor: 'metaclass' stamp: 'cwp 11/4/2005 08:45'!
lookupSelector: aSelector
	
	"Override the implementation in Behavior to make ClassEditor
	debuggable. The debugger calls this during simulated execution,
	and since we override #includesSelector and #superclass to 
	reflect on the subject, we get incorrect method dispatch."

	| lookupClass dict |
	lookupClass := self.
	[lookupClass == nil] whileFalse: 
		[dict := lookupClass instVarAt: 2.
		(dict includesKey: aSelector)
			ifTrue: [^ dict at: aSelector].
		lookupClass := lookupClass instVarAt: 1].
	^ nil! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 9/11/2007 23:52'!
methodDictionary
	| methodEditor |
	(methods isNil and: [self edRequiresRecompile not]) ifTrue: [^ self subject methodDictionary].
	methodEditor := methods ifNil: [MethodDictionaryEditor for: self].
	^ methodEditor buildFor: self! !

!OldMetaclassEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 23:32'!
methods
	^ methods ifNil: [methods := MethodDictionaryEditor for: self]! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 11/5/2005 14:49'!
name
	^ editor name, ' class'! !

!OldMetaclassEditor methodsFor: 'metaclass' stamp: 'cwp 10/23/2005 14:43'!
new
	^ editor := self basicNew! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 1/14/2006 18:03'!
organization
	^ organization ifNil: [organization := OrganizationEditor for: self]! !

!OldMetaclassEditor methodsFor: 'printing' stamp: 'cwp 10/23/2005 14:45'!
printOn: aStream
	aStream
		nextPutAll: 'a MetaclassEditor on: ';
		print: editor subject ! !

!OldMetaclassEditor methodsFor: 'building' stamp: 'mtf 6/16/2008 21:06'!
product
	^ editor product class! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'gk 9/3/2007 23:42'!
rearrangeInstVarNames: associations
	"The associations are name -> pos and cover the subset
	of vars that should be moved. The old just fill in the holes."

	instVarNames := ClassEditor rearrangeVarNames: associations oldNames: instVarNames! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'cwp 10/27/2005 08:31'!
removeInstVarName: aString 
	instVarNames ifNil: [instVarNames := self subject instVarNames].
	instVarNames := instVarNames copyWithout: aString! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'mtf 9/11/2007 23:52'!
removeSelector: aSymbol 
	methods ifNil: [methods := MethodDictionaryEditor for: self].
	methods remove: aSymbol.! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 12/19/2005 23:58'!
sourceCodeAt: aSelector
	^ self subjectDo: [:subject | subject sourceCodeAt: aSelector]! !

!OldMetaclassEditor methodsFor: 'accessing' stamp: 'cwp 12/19/2005 23:56'!
subjectDo: aBlock
	^ self subject
		ifNotNilDo: [:subject | aBlock value: subject]
		ifNil: [self error: 'key not found']! !

!OldMetaclassEditor methodsFor: 'accessing' stamp: 'cwp 11/5/2005 14:42'!
subject
	^ editor subject ifNotNilDo: [:subject | subject class]! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 9/11/2007 23:49'!
superclass
	| subjectSuperclass |
	superEditor ifNotNil: [^ superEditor].
	subjectSuperclass := self subject
		ifNil: [Object]
		ifNotNil: [self subject superclass].
	^ self system edEditorFor: subjectSuperclass! !

!OldMetaclassEditor methodsFor: 'editing' stamp: 'cwp 11/20/2005 16:36'!
superclass: aClass 
	superEditor := self system edEditorFor: aClass! !

!OldMetaclassEditor methodsFor: 'accessing' stamp: 'cwp 11/20/2005 16:34'!
system
	^ editor system! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 9/13/2007 19:52'!
theMetaClass
	^ self! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 9/13/2007 19:53'!
theNonMetaClass
	^ editor! !

!OldMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 7/22/2008 12:37'!
typeOfClass
	^ #normal! !

OldMetaclassEditor subclass: #OldRootMetaclassEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!OldRootMetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/5/2008 23:48'!
canEdit: anObject for: anEditor
	^ false! !

!OldRootMetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/5/2008 23:49'!
classClassEditor
	^ RootClassEditor! !

!OldRootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:48'!
allInstVarNames
	^ #()! !

!OldRootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:59'!
allSuperclasses
	^ OrderedCollection new! !

!OldRootMetaclassEditor methodsFor: 'initialize-release' stamp: 'mtf 8/5/2008 23:48'!
edRegisterEditor
"Do nothing"! !

!OldRootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/5/2008 23:48'!
edRequiresBuild
	^ false! !

!OldRootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/5/2008 23:48'!
edRequiresRecompile
	^ false! !

!OldRootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:48'!
instSize
	^ 0! !

!OldRootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/5/2008 23:48'!
product
	^ nil! !

!OldRootMetaclassEditor methodsFor: 'accessing' stamp: 'mtf 8/5/2008 23:48'!
subject
	^ nil! !

!OldRootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:48'!
typeOfClass
	^ #normal! !

!OldRootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:55'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
"Return the source code at selector, deferring to superclass if necessary"

	^ aBlock value! !

PackageInfo subclass: #SystemEditorInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Info'!

!SystemEditorInfo class methodsFor: 'as yet unclassified' stamp: 'cwp 11/5/2005 17:02'!
testClasses
	^ self new classes select: [:ea | ea allSuperclasses includes: TestCase]! !

!SystemEditorInfo class methodsFor: 'as yet unclassified' stamp: 'cwp 11/5/2005 17:01'!
testSuite
	| suite |
	suite := TestSuite new.
	suite name: self name asString.
	self testClasses do: 
		[:each | 
		each isAbstract ifFalse: [each addToSuiteFromSelectors: suite]].
	^ suite! !

SharedPool subclass: #EditorSubjectPool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

Object subclass: #EditorSubject
	instanceVariableNames: 'alpha beta'
	classVariableNames: 'Beth Aleph'
	poolDictionaries: 'EditorSubjectPool'
	category: 'SystemEditor-Tests'!

EditorSubject class
	instanceVariableNames: 'omega'!

EditorSubject class
	instanceVariableNames: 'omega'!

!EditorSubject class methodsFor: 'accessing class vars' stamp: 'cwp 10/24/2005 18:38'!
aleph
	^ Aleph ! !

!EditorSubject class methodsFor: 'accessing class vars' stamp: 'cwp 10/24/2005 18:38'!
aleph: anObject
	Aleph := anObject! !

!EditorSubject class methodsFor: 'methods for editing' stamp: 'cwp 10/28/2005 08:18'!
ayin
	^ 22! !

!EditorSubject methodsFor: 'phoenician' stamp: 'cwp 10/26/2005 08:41'!
het
	^ 'first letter of the phoenician alphabet'! !
ClassEditor initialize!
